function gtCopy(fnamein, fnameout, fnameout_orig, parameters, move_data_flag)
% GTCOPY  Copies a file from fnamein to fnameout
%     gtCopy(fnamein, fnameout, fnameout_orig, parameters, move_data_flag, varargin)
%     ------------------------------------------------------------------------------
%     INPUT:
%       fnamein        = input file name
%       fnameout       = output file name
%       fnameout_orig  = output original file name
%       parameters     = parameters file
%       move_data_flag = flag for moving data
%     OUTPUT:
%       --none--
%
%     Version 002 28-11-2011 by LNervo
%       *** clean version ***
%
%     Version 001
%       Either move or copy the original file to fnameout_orig
%       Adds to edf header info according to parameters
%
%       Use move_data_flag=0 if the data is already in the 0_rawdata/Orig
%       directory, and shouldn't be moved again.
%
%       varargin - to pass in distortion file if required
%
%

disp(['gtCopy of "' fnamein '"'])

if strcmp(fnamein(end-2:end),'edf')
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % image corrections for edf files
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [im, info] = edf_wait_read(fnamein);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % ccd specific corrections
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    switch lower(parameters.acq.sensortype)
        case 'kodak4mv1'
            %add fields to info
            if ( (isfield(info, 'isccdcorrected') && strcmpi(info.isccdcorrected, true)) ...
                 || (isfield(info, 'israw') && strcmpi(info.israw, 'false')) )
                disp('Kodak corrections already applied')
            else
                disp('applying Kodak chip corrections')
                info.ccdmodel = 'Kodak4Mv1';
                info.isccdcorrected = true;
                % do the kodak chip corrections
                midline=round(parameters.acq.ydet/2);
                im=[im(1:midline,:); ...
                    mean([im(midline,:);im(midline+1,:)]);...
                    im(midline+1:end-1,:)];
                % im=im.^0.96;
            end
        case {'frelon', 'frelon hd2k', 'frelon hde2v'}
            %add fields to info
            info.ccdmodel = parameters.acq.sensortype;
        otherwise
            fprintf('Sensor type: %s', parameters.acq.sensortype)
            gtError('SETUP:copy_error', 'Sensor type "%s" is not recognized.', parameters.acq.sensortype)
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % distortion correction
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~strcmp(parameters.acq.distortion, 'none')
        if isfield(info, 'isundistorted') && strcmpi(info.isundistorted, true)
            disp('image has already been corrected')
            im_corr=im;
        else
          try
            % get distortion map - passed in or read
            disp(['reading distortion map from file: ' ...
                  parameters.acq.distortion])
            detector_size = [];
            detector_size.x = parameters.acq.true_detsizeu;
            detector_size.y = parameters.acq.true_detsizev;
            if ( detector_size.x == parameters.acq.xdet ...
                  && detector_size.y == parameters.acq.ydet )
                disp('Using full detector')
                distortion = distortion_read(parameters.acq.distortion, detector_size);
            else
                disp(['Using a ROI of the detector (' ...
                      num2str(parameters.acq.xdet) ', ' ...
                      num2str(parameters.acq.ydet) ')'])
                % ROI is of the form [xmin ymin x_size y_size]
                roi = [ parameters.acq.detroi_u_off parameters.acq.detroi_v_off ...
                        parameters.acq.xdet parameters.acq.ydet ];
                distortion = distortion_read(parameters.acq.distortion, detector_size, roi);
            end
            disp('applying distortion correction')
            im_corr = distortion_correct(im, distortion);
            info.isundistorted = true;
          catch mexc
            errorMsg = [ 'Not able to apply correction from file: "' ...
                         parameters.acq.distortion '".' ];
            gtPrintException(mexc, errorMsg)
            im_corr = im;
          end
        end
    else
        disp('no correction applied')
        im_corr = im;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % turn images for gt coordinate system
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if isfield(parameters.acq, 'rotation_axis') && strcmp(parameters.acq.rotation_axis, 'horizontal')
        if ~isfield(info,'gtrotated') || strcmp(info.gtrotated, 'false')
            if strcmpi(parameters.acq.rotation_direction, 'clockwise')
                disp('rotating 90 degrees counterclockwise (horizontal rotation axis)')
                info.gtrotated = true;
                im_corr = fliplr(im_corr');  % 90 degrees
            elseif strcmpi(parameters.acq.rotation_direction, 'counterclockwise')
                disp('rotating 90 degrees clockwise (horizontal rotation axis)')
                info.gtrotated = true;
                im_corr = (fliplr(im_corr))'; % -90 degrees
            else
                disp('Options are: counterclockwise and clockwise.')
            end
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % flip images for gt coordinate system
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if parameters.acq.flip_images==true
        if ~isfield(info,'gtlrflipped') || strcmp(info.gtlrflipped, 'false')
            disp('flipping LR for gt coordinate system')
            info.gtlrflipped=true;
            im_corr=fliplr(im_corr);
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % write the corrected image and info to the new location
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    edf_write(im_corr,fnameout,info);
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % non-images files: just copy
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    try
        [~, msg] = copyfile(fnamein,fnameout); disp(msg)
    catch mexc
        gtPrintException(mexc)
        mexc1 = GtException('SETUP:copy_error', ['Not able to copy "' fnamein '" to: "' fnameout '".']);
        mexc1 = mexc1.addCause(mexc);
        throw(mexc1);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% move or copy the original file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if move_data_flag
    try
        [~, msg] = movefile(fnamein, fnameout_orig); disp(msg)
    catch mexc %if no permission to mv, copy instead
        errorMsg = ['Not able to move "' fnamein '" to: "' fnameout_orig '".'];
        gtPrintException(mexc, errorMsg)
        disp('Trying to just copy...')
        try
            [~, msg] = copyfile(fnamein, fnameout_orig); disp(msg)
        catch mexc2
            gtPrintException(mexc2)
            mexc1 = GtException('SETUP:copy_error', ['Not able to move or copy "' fnamein '" to: "' fnameout '".']);
            mexc1 = mexc1.addCause(mexc);
            mexc1 = mexc1.addCause(mexc2);
            throw(mexc1);
        end
    end
end

clear s msg
end %end of function

