
function parameters = gtFillCrystFields(parameters, phase_ndx)
%
% fill in the cryst(phase_ndx)
% handle all crystallography stuff in one easy function...
%
% concept - we need certain information about each phase, and we have to get
% this from somewhere.  This could be from a .dat file created by XOP, a
% .cif file from Diamond or somewhere else, or could be supplied from the
% head of the user - we should not be fussy.
%
% The user should not have to copy anything - as long as the files are on
% disk, we should move them to the right place
% For the moment at least, keep the concept of archiving a dat and cif
% file for every dataset in id11/graintracking/
%
% Information to deal with:
%
% We already have:
% energy, max and min two theta values
%
% We need to get:
% {hkl} or {hkil} reflections
% lattice parameters
% spacegroup
%
% Optional, but nice, are:
% calculated intensities
% symmetry operators (although we don't use this info yet...)
%
% We generate ourselves:
% crystal system and lattice type we can get from spacegroup number
% signed hkls we generate by applying the (hard coded) symmetry operators
% d spaces from hkls and lattice parameters
% theta from dspacing and energy
%
%

disp([' ~~~~~~~~~~~~~ Get crystallography information for phase ' num2str(phase_ndx) ' ~~~~~~~~~~~~~ '])

% now fill in this information for this phase
phase_ok=false;
while ~phase_ok
    
    % is this part of a series of measurements?  In this case we can simply
    % copy from another parameters file
    disp(['If you have already measured this material with same geometry and conditions, ' ...
        'you can copy from another parameters file.  If the experimental geometry is different, '...
        'it may be safer to load from a data file prepared for the specific conditions'])
    
    test = input('Do this? [y/n] ', 's');
    if strcmpi(test, 'y') || strcmpi(test, 'yes')
        %%%% get complete information from another parameters file %%%%%
        parameters = sfGetInfoFromParameters(parameters, phase_ndx);
        
    else
        %%%% get information from files or user %%%%%
        
        
        % Get reflection data
        disp('Do you want to load reflection data from .csv or .dat files?')
        test = input('Do this? [y/n] ', 's');
        if strcmpi(test, 'y') || strcmpi(test, 'yes')
            parameters = sfGetReflectionInfoFromDataFiles(parameters, phase_ndx);
        else
            parameters = sfGetReflectionInfoFromUser(parameters, phase_ndx);
        end
        
        % Get crystallographic data
        disp('Do you want to load crystallographic info (lattice pars, symmetry) from a .cif file?')
        test = input('Do this? [y/n] ', 's');
        if strcmpi(test, 'y') || strcmpi(test, 'yes')
            parameters = sfGetCrystInfoFromDataFiles(parameters, phase_ndx);
        else
            parameters = sfGetCrystInfoFromUser(parameters, phase_ndx);
        end
    
        % expand the input data
        parameters = sfExpandData(parameters, phase_ndx);
        
    end
    
    
    
    
    % Check things look okay
    test = input('Do you want to look at the contents of cryst for this phase? [y/n] ', 's');
    if strcmpi(test, 'y') || strcmpi(test, 'yes')
        parameters = sfCheckWithUser(parameters, phase_ndx);
    end
    
    
    % does this look ok?
    test = input('Is this phase OK? [y/n] ', 's');
    if strcmpi(test, 'y') || strcmpi(test, 'yes')
        phase_ok=true;
        disp('')
        
        disp('Saving parameters.mat in the analysis directory...')
        parameters_name=fullfile(parameters.acq.dir,'parameters.mat');
        save(parameters_name, 'parameters')
        disp([' ~~~~~~~~~~~~~ Finished getting crystallography information for phase ' num2str(phase_ndx) ' ~~~~~~~~~~~~~ '])
    end
    
end % end while loop


end % end of the main function


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% SUB FUNCTIONS

function parameters=sfCheckWithUser(parameters, ii)
% show the user the cryst field so they can decide if it looks ok
parameters.cryst(ii)

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%          COPY FROM A PARAMETERS.MAT FILE             %%%%%%%%%%%%%%
% in this case, will fill cryst directly
function parameters = sfGetInfoFromParameters(parameters, ii)

% get the master parameters
[masterfile, masterpath]=uigetfile('parameters.mat');
if strcmp(masterfile, 'parameters.mat')
    master=load(fullfile(masterpath, masterfile));
    % if there is more than one phase
    if length(master.parameters.cryst)>1
        query='Which phase to use? ';
        for m=1:length(master.parameters.cryst)
            query=[query sprintf('(%d) %s; ', m, master.parameters.cryst(m).name)];
        end
        query=[query '[1/2/3...] '];
        mm=input(query);
    else
        mm=1;
    end
    % add these values to our parameters field
    parameters.cryst(ii) = gtAddMatFile(parameters.cryst(ii),master.parameters.cryst(mm),true,true,true);
    % keep track of where we got the data from
    
    % create the xop directory in which to save the files used
    [xop_dir,msg] = gtCrystXopInitializeDirectory(parameters.acq.name, parameters.cryst(ii).name);
    parameters.xop(ii).xop_dir=xop_dir;
    parameters.xop(ii).filename='master_parameters.mat';
    % copy the parameters file that we used as the reference
    copyfile(fullfile(masterpath, masterfile), fullfile(xop_dir, 'master_parameters.mat'))
    % return to the main loop to allow user to check manually
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%     LOAD FROM A DATA FILEs (.CSV or .DAT, AND .CIF)      %%%%%%%%
% will fill xop with "raw" data
function parameters = sfGetReflectionInfoFromDataFiles(parameters, ii)

% this is a "new" experiment, set up the names and read from the datafile
disp(['Before loading, please give a description for this phase (' num2str(ii) ')'])
% get this stuff via gtModfiyStructure
biglist=build_list_v2;
list=biglist.cryst;
listndx=find( strcmp(list(:,1), 'name') | strcmp(list(:,1), 'composition') | strcmp(list(:,1), 'material'));
list=list(listndx, 1:2);
parameters.cryst(ii)=gtModifyStructure(parameters.cryst(ii), list, sprintf('Manual input of crystallographic parameters for phase %d', ii));

% create the xop directory in which to save the files used
[xop_dir,msg] = gtCrystXopInitializeDirectory(parameters.acq.name, parameters.cryst(ii).name);

% get a data file to try
disp('Please select a data file from which to load powder diffraction data')
% pick a suitable datafile
[datafile, datapath]=uigetfile('*.csv; *.dat', 'Select a powder diffraction data file', '/data/id11/graintracking/');
% try to read from it
try
    xop = gtLoadReflectionsFromFile(fullfile(datapath, datafile));
    parameters.xop(ii) = gtAddMatFile(parameters.xop(ii),xop,true,true,false);
    clear xop
    % copy the file that we used
    copyfile(fullfile(datapath, datafile), fullfile(xop_dir, datafile))
    parameters.xop(ii).xop_dir=xop_dir;
    parameters.xop(ii).filename=datafile;
    disp('read some powder diffraction data')
    % add this data to parameters.cryst
    
catch Mexc
    disp('Did not manage to read frlom the file you chose!')
    disp('keep trying - possibly with a different datafile or manual input')
    return
end
end % end sfGetReflectionInfoFromDataFiles
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function parameters = sfGetCrystInfoFromDataFiles(parameters, ii)
% need to get spacegroup and lattice parameters from a .cif file
% this goes into parameters.cryst

[ciffile, cifpath]=uigetfile('*.cif', 'Select a suitable crystallography file', '/data/id11/graintracking/');
cif = gtCrystCifInitialize(fullfile(cifpath, ciffile));
parameters.cryst(ii) = gtAddMatFile(parameters.cryst(ii),cif,true,true,false);
clear cif
% copy the file that we used to the xop_dir
copyfile(fullfile(cifpath, ciffile), fullfile(parameters.xop(ii).xop_dir, ciffile))

end % read of sfGetCrystInfoFromDataFiles
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function parameters = sfGetReflectionInfoFromUser(parameters, ii)
% get info interactively

disp(['Please give a description for this phase (' num2str(ii) ')'])

% We need to get:
% {hkl} or {hkil} reflections
% lattice parameters
% spacegroup
% get the list for the fields we are interested in
% in parameters.cryst
biglist=build_list_v2();
list=biglist.cryst;
listndx=find( strcmp(list(:,1), 'name') | strcmp(list(:,1), 'composition') | strcmp(list(:,1), 'material'));
list=list(listndx, 1:2);
parameters.cryst(ii)=gtModifyStructure(parameters.cryst(ii), list, sprintf('Manual input of reflections for phase %d', ii));

% create the xop directory in which to save the files used
[xop_dir,msg] = gtCrystXopInitializeDirectory(parameters.acq.name, parameters.cryst(ii).name);
% no datafile to copy in this case

% in parameters .xop
hkl=input('Manually input the hkl families as an nx3 matrix: [h1 k1 l1; h2 k2 l2; etc]');
parameters.xop(ii).hkl=hkl;
disp('filling parameters.xop.int and mult with ones')
parameters.xop(ii).int=ones(size(hkl));
parameters.xop(ii).mult=ones(size(hkl));

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function parameters = sfGetCrystInfoFromUser(parameters, ii)
% need to get spacegroup and lattice parameters from a .cif file
% We need to get:
% lattice parameters
% spacegroup
% get the list for the fields we are interested in
% in parameters.cryst
biglist=build_list_v2();
list=biglist.cryst;
listndx=find( strcmp(list(:,1), 'latticepar') | strcmp(list(:,1), 'spacegroup'));
list=list(listndx, 1:2);
parameters.cryst(ii)=gtModifyStructure(parameters.cryst(ii), list, sprintf('Manual input of crystallographic parameters for phase %d', ii));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function parameters = sfExpandData(parameters, ii)
% file parameters.cryst from reflection data in parameters.xop and symmetry
% and lattice parameters in parameters.cryst

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get crystal system and lattice from the spacegroup number
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[lattice_system, crystal_system, ~] = gtReadSpaceGroup(parameters.cryst(ii).spacegroup);
parameters.cryst(ii).lattice_system = lattice_system;
parameters.cryst(ii).crystal_system = crystal_system;
clear lattice_system crystal_system

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% getting reflection on the detector, basing on theta max and min
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% after xop
% this cleans the xop output into distinct hkl families
% and recalculates dspacings and thetas according to parameters
% discards anything not on the detector
% this info goes into the parameters.cryst field
detector = gtDetectorTwotheta(parameters, ii);
parameters.cryst(ii) = gtAddMatFile(parameters.cryst(ii),detector,true,true,false);
clear detector

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Using symmetry operators getting all the possible reflections
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this expands the hkl families in parameters.cryst
hklsymmetry = gtSymmetricReflections(parameters.cryst(ii),parameters.acq.energy);
parameters.cryst(ii) = gtAddMatFile(parameters.cryst(ii),hklsymmetry,true,true,false);
clear hklsymmetry

% Transpose arrays to have rows
parameters.cryst(ii).hkl       = parameters.cryst(ii).hkl';
parameters.cryst(ii).thetatype = parameters.cryst(ii).thetatype';
parameters.cryst(ii).theta     = parameters.cryst(ii).theta';
parameters.cryst(ii).dspacing  = parameters.cryst(ii).dspacing';
parameters.cryst(ii).int       = parameters.cryst(ii).int';
parameters.cryst(ii).mult      = parameters.cryst(ii).mult';

parameters.cryst(ii).hklsp       = parameters.cryst(ii).hklsp';
parameters.cryst(ii).thetatypesp = parameters.cryst(ii).thetatypesp';
parameters.cryst(ii).thetasp     = parameters.cryst(ii).thetasp';
parameters.cryst(ii).dspacingsp  = parameters.cryst(ii).dspacingsp';
parameters.cryst(ii).intsp       = parameters.cryst(ii).intsp';
end
