function twotheta=calculate_twotheta(d0,spacegroup,energy,strain)
% CALCULATE_TWOTHETA.M
% Given the d0 spacing, spacegroup, and energy, returns the diffraction
% angles for different reflections.
%
% Usage:
% theta=calculate_theta(d0,spacegroup,energy[,strain])
%
% Input:
% d0 is lattice spacing in angstroms (eg 4.04)
% spacegroup is 'FCC' (only option supported at present
% energy is in keV
% strain, if present, is between 0 and 1.
%
% Output:
% twotheta is a mhashtable object - an array which has a NAME for each
% element, not an index.  You can get look at your results like this:
% if twotheta.containsKey('111')
%   twotheta.get('111')
% end
% the min and max values only appear if the strain is non-zero
% More help on mhashtable in demo_hashtable


% Check parameters
if ~ischar(spacegroup)
  error('Spacegroup must be a string - ''FCC'',''BCC'',''HCP''');
end

if ~exist('strain','var')
  % only calculate perfect values
  strain=0;
elseif strain<0|strain>1
  error('Strain must be between 0 and 1')
end


switch spacegroup
  case 'FCC'
    % verify the theoretically allowed reflections !
    reflections=[...
      1 1 1;...
      2 0 0;...
      2 2 0;...
      3 1 1;...
      4 0 0;...
      3 3 1;...
      ];

    lambda=12.398/energy;   % verify constant !
    twotheta=mhashtable;
    for i=1:size(reflections,1)
      
      tmp=2*180/pi*asin(lambda/2./(d0*[1-strain 1 1+strain])*norm(reflections(i,:)));
      if strain~=0
        vals.max=tmp(1);
        vals.min=tmp(3);
      end
      vals.centre=tmp(2);
      stmp=num2str(reflections(i,:));
      stmp(isspace(stmp))=[];
      twotheta.put(stmp,vals);
    end
  otherwise
    error('Sorry, only FCC possible at present');
end


end




