function sample = gtSolveGrainsConflicts(minnum, save_flag)
% GTSOLVEGRAINSCONFLICTS
%     sample = gtSolveGrainsConflicts(minnum, save_flag)
%     --------------------------------------------------
%     To be run in the analysis directory after gtUpdateGrainsConflicts
%
%     Reads ./4_grains/sample.mat
%           ./4_grains/spot2grain.mat
%
%     Write ./4_grains/out.mat
%        out{phaseid}.deselected.grains     = deselected grains
%        out{phaseid}.deselected.difspots   = difspots in conflict of the deselected grains
%        out{phaseid}.modified.grains       = modified grains with summary{grainid}.totspots < minnum
%        out{phaseid}.modified.difspots     = corresponding difspots delected from those grains
%        out{phaseid}.conflicts.grains      = grainid still in conflicts
%        out{phaseid}.spot2grain            = {spot2grain, spot2phase};
%
%     INPUT:
%       minnum    = minimum number of spots to be discarded <double> {4}
%       save_flag = flag to save 4_grains/sample.mat and 4_grains/spot2grain.mat 
%                   <logical> {false}
%
%     OUTPUT:
%       sample = updated sample.mat <struct>
%
%     Version 003 15-06-2012 by LNervo
%       sample.phases{phaseid}.summary{grainid}.flag can be:
%          0 - no conflict
%          1 - conflict and threshold < 0
%          2 - conflict and (completeness - mean) < 3*std
%          3 - conflict and totspots < maxnum


if ~exist('minnum','var') || isempty(minnum)
    minnum = 4;
end
if ~exist('save_flag','var') || isempty(save_flag)
    save_flag = false;
end

out        = [];
sample     = [];
spot2grain = [];
spot2phase = [];

filedir = '4_grains';
bkpdir  = fullfile('4_grains','archive');
fname0  = fullfile(bkpdir,'%s');

sample_root = fullfile(filedir,'sample.mat');
spot_root   = fullfile(filedir,'spot2grain.mat');
out_root    = fullfile(filedir,'out.mat');

sampleFile = gtLastFileName(sprintf(fname0,['sample_' date() '_']),'new');
spotFile   = gtLastFileName(sprintf(fname0,['spot2grain_' date() '_']),'new');
outFile    = gtLastFileName(sprintf(fname0,['out_' date() '_']),'new');

load(sample_root);
load(spot_root);
num_phases = length(sample.phases);

difspots_list = [];


for phaseid = 1 : num_phases
    phase = sample.phases{phaseid};
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get grains with threshold < 0
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_thr = phase.getGrainsWithFieldValueLessThan('threshold', 0);
    % set phase.selectedGrains = false
    arrayfun(@(array) phase.setSelected(array, false), grainIDs_thr);
    % set summary.flag = 1
    arrayfun(@(array) phase.setFlag(array, 1), grainIDs_thr);
    disp(['Deselected ' num2str(length(grainIDs_thr)) ' grains with threshold < 0'])

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get grains with (completeness - mean ) < 3*std
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    thr_com = mean(phase.completeness(:))-3*std(phase.completeness(:));
    grainIDs_com = phase.getGrainsWithCompletenessLessThan(thr_com)'; % row vector
    % set phase.selectedGrains = false
    arrayfun(@(array) phase.setSelected(array, false), grainIDs_com);
    % set summary.flag = 2
    arrayfun(@(array) phase.setFlag(array, 2), grainIDs_com);
    disp(['Deselected ' num2str(length(grainIDs_com)) ' grains with (completeness - mean) < 3*std ( = ' num2str(thr_com) ')'])

    clear thr_com

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % get deselected grains
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs_desel = phase.getDeselectedGrains();
    % set delete = true
    arrayfun(@(array) phase.setDelete(array, true), grainIDs_desel);
    %     disp(['Set summary.delete = true for ' num2str(length(grainIDs_desel)) ' grains'])
    
    % get grainIDs with hasConflicts=true
    hasC = arrayfun(@(array) phase.getHasConflicts(array), grainIDs_desel);
    if ~isempty(grainIDs_desel(hasC))
        % set idspot for grainIDs_desel(hasC)
        idspot = arrayfun(@(array) phase.buildIdSpot(array), grainIDs_desel(hasC), 'UniformOutput', false);
        %     disp(['Set summary.idspot for ' num2str(length(grainIDs_desel(hasC))) ' grains'])
        
        % update spot2grain and spot2phase
        % create difspots_list and grains_list (only grains with conflicts)
        list_desel = [idspot{:}];
        grains_list_desel = grainIDs_desel(hasC);
        if ~isempty(list_desel)
            difspots_list_desel = list_desel;
            output_desel = sfUpdateSpot2Grain(grains_list_desel, difspots_list_desel);
            
            if save_flag
                copyfile(spot_root, spotFile);
                save(spot_root,'-struct','output_desel');
                disp(['Updated ' spot_root])
            end
            
            spot2grain = output_desel.spot2grain;
            spot2phase = output_desel.spot2phase;
        end
        
        
        % for each deselected grain with conflict, get idconflict
        conflicts = phase.getAllValuesFromSummaryField('idconflict',false);
        [idconflicts_desel, indexes_desel] = sfgetSinglesValues(conflicts, grains_list_desel);
    else
        idconflicts_desel = [];
        indexes_desel     = [];
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % deselect difspots in conflict from grains{grainid} still in conflicts
    % if difspots <= maxnum
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    grainIDs = find(phase.getSelected(:)==true & phase.getHasConflicts(:)==true)'; % row vector
    % set idspot for grainIDs with conflicts and selected
    idspot = arrayfun(@(array) phase.buildIdSpot(array), grainIDs, 'UniformOutput', false);
%     disp(['Set summary.idspot for ' num2str(length(grainIDs)) ' grains'])
    
    % difspots <= maxnum
    grainIDs_max = phase.getGrainsWithNofConflictsLessThan(minnum)'; % row vector
    % get all the grains.selectedDiffspots
    disp(['Found ' num2str(length(grainIDs_max)) ' grains with less than ' num2str(minnum) ' difspots in conflict'])
    
    if ~isempty(grainIDs_max)
        idspot_max = arrayfun(@(array) phase.buildIdSpot(array), grainIDs_max, 'UniformOutput', false);
        % get indexes for each grain with difspots in conflict
        indexes_max = arrayfun(@(array) phase.getIdSpotIndexes(array), grainIDs_max, 'UniformOutput', false);
        % get ids for each grain with difspots in conflict
        idspots_max = arrayfun(@(array) phase.getIdSpotIds(array), grainIDs_max, 'UniformOutput', false);
        % for each modified grain with conflict, get idconflict
        idconflict_max = arrayfun(@(array) phase.getIdConflict(array), grainIDs_max, 'UniformOutput', false);
        
        % set to false in grains{grainid}.selectedDiffspots
        arrayfun(@(num) phase.setSelectedDiffspotsIndexes(grainIDs_max(num), indexes_max{num}, false), 1:length(indexes_max));
        disp(['Deselected the corresponding diffspots in conflict for ' num2str(length(grainIDs_max)) ' grains'])
        arrayfun(@(num) disp(['Deselected ' num2str(length(indexes_max{num})) ' difspots from grain ' num2str(grainIDs_max(num))]), 1:length(indexes_max));
        % set summary.flag = 3
        arrayfun(@(array) phase.setFlag(array, 3), grainIDs_max);
        % set summary.delete = false & summary.delete
        arrayfun(@(array) phase.setDelete(array, false & phase.getDelete(array)), grainIDs_max);
        
        disp('Handles conflicts of modified grains')
        id_conflict_max = [idconflict_max{:}];
        % get only grainids which have less than minnum difspots
        [~, ind] = intersect(id_conflict_max,grainIDs_max);
        id_c_max = id_conflict_max(ind);
        
        idspot_conflict_max = arrayfun(@(array) phase.buildIdSpot(array), id_c_max, 'UniformOutput', false);
        % get indexes for each grain in conflict with grainIDs_max with
        % less than minnum difspots
        indexes_conflict_max = arrayfun(@(array) phase.getIdSpotIndexes(array), id_c_max, 'UniformOutput', false);
        % set to false in grains{grainid}.selectedDiffspots
        arrayfun(@(num) phase.setSelectedDiffspotsIndexes(id_c_max(num), indexes_conflict_max{num}, false), 1:length(indexes_conflict_max));
        disp(['Deselected the corresponding diffspots in conflict for ' num2str(length(id_c_max)) ' grains'])
        arrayfun(@(num) disp(['Deselected ' num2str(length(indexes_conflict_max{num})) ...
                              ' difspots from grain ' num2str(id_c_max(num))]), 1:length(indexes_conflict_max));
        % set summary.flag = 3
        arrayfun(@(array) phase.setFlag(array, 3), id_c_max);
        
        % update spot2grain and spot2phase
        list_max = [idspot_max{:}, idspot_conflict_max{:}];
        grains_list_max = [grainIDs_max, id_c_max];
        if ~isempty(list_max)
            difspots_list_max = list_max;
            output_max = sfUpdateSpot2Grain(grains_list_max, difspots_list_max);
            spot2grain = output_max.spot2grain;
            spot2phase = output_max.spot2phase;
            
            if save_flag
                copyfile(spot_root, spotFile);
                save(spot_root,'-struct','output_max');
                disp(['Updated ' spot_root])
            end
        end
        
        % for each modified grain with conflict, get idconflict
        conflicts = phase.getAllValuesFromSummaryField('idconflict',false);
        [idconflicts_max, indexes_c_max] = sfgetSinglesValues(conflicts, grains_list_max);
 
    else
        idconflicts_max = [];
        indexes_c_max   = [];
    end
        
    % get remaining conflicts
    conflicts = phase.getAllValuesFromSummaryField('idconflict',false);
    [conflicts, indexes_c] = sfgetSinglesValues(conflicts, []);
    
    conflicts_out = [indexes_c, conflicts];
    conflicts_unique = unique(sort(conflicts_out,2),'rows');
    conflicts_list = unique(sort(conflicts_out,1));
    
    % saving information
    
    out{phaseid}.deselected_grains = [indexes_desel, idconflicts_desel];
    out{phaseid}.modified_grains   = [indexes_c_max, idconflicts_max];
    out{phaseid}.conflicts_grains  = [indexes_c, conflicts];
    out{phaseid}.conflicts_list    = conflicts_list;
    out{phaseid}.conflicts_unique  = conflicts_unique;
    out{phaseid}.spot2grain        = {spot2grain, spot2phase};

    sample.phases{phaseid} = phase;

end % end for phaseid

if exist(out_root,'file')
    copyfile(out_root, outFile);
end
save(out_root,'out', '-v7.3');
disp(['Saved output in ' out_root])

if save_flag
    copyfile(sample_root, sampleFile);
    save(sample_root, 'sample', '-v7.3');
    disp(['Updated ' sample_root])
end

end % end of function


%% 
% sub-functions
function output = sfUpdateSpot2Grain(grains_list, difspots_list)

spot2grain = [];
spot2phase = [];
load(fullfile('4_grains','spot2grain.mat'));
% update spot2grain and spot2phase
for ii = 1 : length(grains_list)
    for jj = 1 : length(difspots_list)
        index = spot2grain{difspots_list(jj)}==grains_list(ii);
        spot2grain{difspots_list(jj)}(index)=[];
        spot2phase{difspots_list(jj)}(index)=[];
    end
end
disp(['Updated spot2grain.mat for ' num2str(length(difspots_list)) ' difspots in conflict with ' num2str(length(grains_list)) ' grains'])
output.spot2grain = spot2grain;
output.spot2phase = spot2phase;

end % end of sfUpdateSpot2Grain

function [allvalues, grainid] = sfgetSinglesValues(cellArray, selected)

allvalues = [];
grainid   = [];
count = 0;

if ~isempty(selected)
    cellArray = arrayfun(@(num) cellArray{num}, selected, 'UniformOutput', false);
else
    selected = 1:length(cellArray); 
end

for i = 1 : length(cellArray)
    array = cellArray{i};
    if ~isempty(array)
        for j = 1 : length(array)
            current = array(j);
            if isempty(current)
                count = count + 1;
            else
                grainid = [grainid; selected(i)];
                allvalues = [allvalues; current];
            end
        end
    end
end
            
end % end of sfgetSinglesValues
