function grain = gtThresholdGrain3D(grain, rec)
% GTTHRESHOLDGRAIN3D segment 3D grain volume
%
% function grain = gtThresholdGrain3D(grain, rec)
%
% INPUTS:  grain.vol          graylevel reconstruction of grain volume
%          rec                contents of parameters.rec structure:
%             .percentile2    adaptive threshold see below
%             .do_morph_recon remove not connected areas by morphological
%                             reconstruction (<logical> {true})
% OUTPUTS:
%          grain.thresh Threshold value
%          grain.seg    Segmented grain volume
%          grain.Area   Area of grain
%          grain.segbb  BoundingBox (gt convention) for insertion of seg
%                       volume into sample volume
%
% explanation - we expect a histogram with a peak corresponding to the
% grain, and a peak a low values corresponding to the background of zeros.
% We want a threshold value below the peak corresponding to the grain, but
% above the background peak.

vol     = grain.vol;
grainid = grain.id;

% calulate intensity in the central region of the grain
centerx = round([size(vol,1) * 0.45 : size(vol,1) * 0.55]);
centery = round([size(vol,2) * 0.45 : size(vol,2) * 0.55]);
centerz = round([size(vol,3) * 0.45 : size(vol,3) * 0.55]);

background = gtImgMeanValue(vol(centerx, centery, centerz)) / 10;
% Small performance boost
indexes = vol > background;
% define threshold as percentile/100 x standard dev below the mean, excluding low (10 percentile background) values
stdvol  = std( vol(indexes) );
meanvol = mean( vol(indexes) );

if stdvol > meanvol
    threshold_val = rec.percentile1/100 * meanvol;
else    
    threshold_val = meanvol - rec.percentile2/100 * stdvol;
end

segvol = vol > threshold_val;

% while any(size(segvol) == size(vol))
%     threshold_val = threshold_val + 0.1    
%     segvol = vol > threshold_val;
% end

% morphological reconstruction removes not connected areas
if (rec.do_morph_recon)
    try
        marker = false(size(vol));
        marker(centerx,centery,centerz) = true;
        marker = marker & segvol;
        segvol = double(imreconstruct(marker, segvol));
    catch mexc
        message = sprintf('morphological reconstruction failed for grain %d - continue without...', grainid);
        gtPrintException(mexc, message);
    end
end

regprop = regionprops(segvol, 'BoundingBox', 'Area');

if isempty(regprop)
    segbb       = [];
    grain.seg   = [];
    grain.Area  = [];
    grain.segbb = [];    
else
    segbb = regprop.BoundingBox([2 1 3 5 4 6]);
    
    grain.seg   = gtCrop(segvol, ceil(segbb));    % ceil because Matlab regionprops yields "0.5" (instead of 1) for bb start values
    grain.Area  = regprop.Area;
    segbb(1:3)  = grain.shift + floor(segbb(1:3));
    grain.segbb = segbb;
end

grain.threshold = threshold_val;

end
