function dsp = gtCrystDSpacing(hkl,lp)

% Calculates d-spacing for given hkl-s and lattice parameters.
% 
% INPUT  hkl - set of Miller indices (3,n) or (4,n)
%        lp  - lattice parameters (1,6)
%
% OUTPUT dsp - row vector of d-spacings


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Lattice vectors in real space, in a Cartesian basis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Crystal lattice vectors: a1, a2, a3 (not orthogonal).
% Assume an arbitrary orthogonal base (X,Y,Z).
% Place 'a' vectors in (X,Y,Z) such that:
%   a1 is set along X
%   a2 is set in the X-Y plane

% Calculate 'a1' and 'a2' lattice vector X,Y,Z coordinates:
a1 = [lp(1)             0                 0];
a2 = [lp(2)*cosd(lp(6)) lp(2)*sind(lp(6)) 0];

% Calculate 'a3' components in X-Y plane:
x3 = lp(3)*cosd(lp(5));
y3 = lp(3)*(cosd(lp(4))-cosd(lp(5))*cosd(lp(6)))/sind(lp(6));

% Calculate 'a3' Z coordinate from length
z3 = sqrt(lp(3)^2 - x3^2 - y3^2);

% Assemble 'a3'
a3 = [x3 y3 z3];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Reciprocal lattice vectors in the Cartesian basis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reciprocal lattice vectors by definition:
%  b1 = cross(a2,a3)/avol
%  b2 = cross(a3,a1)/avol
%  b3 = cross(a1,a2)/avol

% Volume of spanned by 'a'-s (determinant of matrix of a-s)
crossa2a3 = [a2(2)*a3(3)-a2(3)*a3(2); a2(3)*a3(1)-a2(1)*a3(3); a2(1)*a3(2)-a2(2)*a3(1)];
avol = a1*crossa2a3;

b1 = crossa2a3/avol;
b2 = [a3(2)*a1(3)-a3(3)*a1(2); a3(3)*a1(1)-a3(1)*a1(3); a3(1)*a1(2)-a3(2)*a1(1)]/avol;
b3 = [a1(2)*a2(3)-a1(3)*a2(2); a1(3)*a2(1)-a1(1)*a2(3); a1(1)*a2(2)-a1(2)*a2(1)]/avol;

% If hexagonal material, transform the four indices into three
if size(hkl,1) == 4
    hkl = hkl([1 2 4],:);
end

% Reciprocal lattice point in the Cartesian basis (a vector normal to hkl plane)
ghkl = [b1 b2 b3]*hkl;

% D-spacings
dsp = 1./sqrt(sum(ghkl.*ghkl,1));



% % To be checked against:
% % Lattice base vectors in rows:
% av = [a1; a2; a3]
% % Reciprocal space vectors in coloumns:
% bv = inv(av)
% ghkl = bv*hkl
% dsp = 1/norm(ghkl)


