function [allshkls, allhklinds, mult]  = gtCrystSignedHKLs(hkl,spacegroup)
% GTCRYSTSIGNEDHKLS  Gives a complete list of signed hkl planes, by
%     applying symmetry operators of the specified spacegroup to the input
%     list of plane families.
%     Should be revised for at least other than cubic lattices. Use of 
%     'lattice_system' and 'crystal_system' instead of spacegroups should
%     be considered.
%
%     [allshkls, allhklinds, mult] = gtCrystSignedHKLs(hkl,spacegroup)
%     ----------------------------------------------------------------
%
%     INPUT:
%       hkl        = plane families (one in a row)
%                    nx3,class double for cubic lattices 
%                    nx4,class double for hexagonal lattices
%       spacegroup = space group
%
%     OUTPUT:
%       allshkls   = complete list of signed hkl planes
%                    (including Friedel pairs)
%       allhklinds = indices in allshkls according to input hkl 
%       mult       = multiplicity of hkl families
%

% Get symmetry operators depending on spacegoup:
switch spacegroup

    case {225, 227, 229, 221} % Cubic
        sym = gtCrystSymOpCubic() ; % 3x3 matrix

    case {152, 167, 194, 663} % Hexagonal
        sym = gtCrystSymOpHexagonal(spacegroup) ; % 4x4 or 3x3 matrix

    otherwise
        error('Spacegroup not supported')
end


allshkls   = [];
allhklinds = [];

% Loop through input hkl types
for i = 1:size(hkl,1)

    shkls = [];

    % Apply all symmetry operators to the hkl type to get signed hkl-s
    for j = 1:length(sym)
		shkls(2*j-1,:) =  hkl(i,:)*sym(j).g;
		shkls(2*j,:)   = -shkls(2*j-1,:);  % Friedel pair of previous one
    end

	% Remove duplicates and add to the complete list
    newshkls = unique(shkls, 'rows');
	allshkls = [allshkls; newshkls];
    
    mult(i,1)  = size(newshkls,1);
    
    allhklinds = [allhklinds; repmat(i,size(newshkls,1),1)];

end


end % end of function

