/*
 * internal_cell_defs.h
 *
 *  Created on: Jul 11, 2012
 *      Author: vigano
 */

#ifndef INTERNAL_CELL_DEFS_H_
#define INTERNAL_CELL_DEFS_H_

#include <math.h>
#include <omp.h>
#include "mex.h"

# define ROUND_DOWN(x, s) ((x) & ~((s)-1))

#if defined(__SSE2__) || defined(__SSE2_MATH__)
  typedef double v2df __attribute__ ((vector_size (16)));
#endif

template<typename Type1, typename Type2>
class inner_assign {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData2);
  }
};

template<typename Type1, typename Type2>
class inner_pow {
  const double exponent;
public:
  inner_pow(const double & _exp) : exponent(_exp) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return pow(inData2, exponent);
  }
};

template<typename Type1, typename Type2>
class inner_sqrt {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return sqrt(inData2); // __builtin_ia32_sqrtpd
  }
};

#if defined(__SSE2__) || defined(__SSE2_MATH__)
template<>
class inner_sqrt<const v2df, const v2df> {
public:
  const v2df
  operator()(const v2df & inData1, const v2df & inData2) const throw()
  {
    return __builtin_ia32_sqrtpd(inData2);
  }
};
#endif

template<typename Type1, typename Type2>
class inner_sum {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 + inData2);
  }
};

template<typename Type1, typename Type2>
class inner_prod {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 * inData2);
  }
};

template<typename Type1, typename Type2>
class inner_sub {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 - inData2);
  }
};

template<typename Type1, typename Type2>
class inner_div {
public:
  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 / inData2);
  }
};


template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sum_scale {
  const TypeScalar scale;
public:
  inner_sum_scale(const TypeScalar & _scale) : scale(_scale) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 + scale * inData2);
  }
};

template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sum_FISTA_scale {
  const TypeScalar scale;
public:
  inner_sum_FISTA_scale(const TypeScalar & _scale) : scale(_scale) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 + scale * (inData1 - inData2));
  }
};

template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sub_scale {
  const TypeScalar scale;
public:
  inner_sub_scale(const TypeScalar & _scale) : scale(_scale) { }

  const Type2
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData1 - inData2 * scale);
  }
};


template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_sum_scalar {
  const TypeScalar scalar;
public:
  inner_sum_scalar(const TypeScalar & _scalar) : scalar(_scalar) { }

  const Type1
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData2 + scalar);
  }
};

template<typename Type1, typename Type2, typename TypeScalar = Type1>
class inner_prod_scalar {
  const TypeScalar scalar;
public:
  inner_prod_scalar(const TypeScalar & _scalar) : scalar(_scalar) { }

  const Type1
  operator()(Type1 & inData1, Type2 & inData2) const throw()
  {
    return (inData2 * scalar);
  }
};

template<mxClassID type>
inline void
cell_allocate(mxArray * & outArray, const mxArray * const inArray)
{
  const mwSize numCells = mxGetNumberOfElements(inArray);
  const mwSize numCellDims = mxGetNumberOfDimensions(inArray);
  const mwSize * cellDims = mxGetDimensions(inArray);
  outArray = mxCreateCellArray(numCellDims, cellDims);

  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * inCell = mxGetCell(inArray, cellIdx);
    const mwSize numDims = mxGetNumberOfDimensions(inCell);
    const mwSize * dims = mxGetDimensions(inCell);
    mxArray * const outCell = mxCreateNumericArray(numDims, dims, type, mxREAL);
    mxSetCell(outArray, cellIdx, outCell);
  }
}

#if defined(__SSE2__) || defined(__SSE2_MATH__)
template<const bool allocate, typename Function, typename Function2>
inline void
cell_iteration_sse(mxArray * & outArray, const mxArray * const inArray,
    const Function & func, const Function2 & func2)
{
  if (allocate) {
    cell_allocate<mxDOUBLE_CLASS>(outArray, inArray);
  }

  const mwSize numCells = mxGetNumberOfElements(inArray);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell = mxGetCell(inArray, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell);
      const double * const inData = mxGetPr(inCell);
      double * const outData = mxGetPr(outCell);

      cell_inner_cycle_sse(outData, outData, inData, numElems, func, func2);
    }
  }
}

template<typename Function, typename Function2>
inline void
cell_iteration_copy_sse(mxArray * & outArray, const mxArray * const inArray1,
    const mxArray * const inArray2,
    const Function & func, const Function2 & func2)
{
  cell_allocate<mxDOUBLE_CLASS>(outArray, inArray1);

  const mwSize numCells = mxGetNumberOfElements(inArray1);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
      const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell1);
      const double * const inData1 = mxGetPr(inCell1);
      const double * const inData2 = mxGetPr(inCell2);
      double * const outData = mxGetPr(outCell);

      cell_inner_cycle_sse(outData, inData1, inData2, numElems, func, func2);
    }
  }
}

template<typename Type, typename FunctionVect, typename FunctionNonVect>
inline void
cell_inner_cycle_sse(Type * const __restrict outData,
    const Type * const __restrict inData1,
    const Type * const __restrict inData2,
    const mwSize & numElems, FunctionVect & func, FunctionNonVect & func2)
{
  const mwSize num8Elems = ROUND_DOWN(numElems, 8);
#pragma omp for nowait
  for(mwIndex elemIdx = 0; elemIdx < num8Elems; elemIdx += 8) {
    const v2df inV11 = *((v2df *)&inData1[elemIdx+0]);
    const v2df inV12 = *((v2df *)&inData1[elemIdx+2]);
    const v2df inV13 = *((v2df *)&inData1[elemIdx+4]);
    const v2df inV14 = *((v2df *)&inData1[elemIdx+6]);

    const v2df inV21 = *((v2df *)&inData2[elemIdx+0]);
    const v2df inV22 = *((v2df *)&inData2[elemIdx+2]);
    const v2df inV23 = *((v2df *)&inData2[elemIdx+4]);
    const v2df inV24 = *((v2df *)&inData2[elemIdx+6]);

    *((v2df *)&outData[elemIdx+0]) = func(inV11, inV21);
    *((v2df *)&outData[elemIdx+2]) = func(inV12, inV22);
    *((v2df *)&outData[elemIdx+4]) = func(inV13, inV23);
    *((v2df *)&outData[elemIdx+6]) = func(inV14, inV24);
  }
#pragma omp for nowait
  for(mwIndex elemIdx = num8Elems; elemIdx < numElems; elemIdx++) {
    outData[elemIdx] = func2(inData1[elemIdx], inData2[elemIdx]);
  }
}

inline double
cell_square_norm_sse(const mxArray * const inArray)
{
  double temp_sum = 0;
  v2df accumul = {0, 0};

  const mwSize numCells = mxGetNumberOfElements(inArray);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell = mxGetCell(inArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell);
      const double * const inData = mxGetPr(inCell);

      const mwSize num32Elems = ROUND_DOWN(numElems, 32);

#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = 0; count < num32Elems; count += 32) {
        const v2df in00 = *((v2df *)&inData[count+0]);
        const v2df in02 = *((v2df *)&inData[count+2]);
        const v2df in04 = *((v2df *)&inData[count+4]);
        const v2df in06 = *((v2df *)&inData[count+6]);
        const v2df in08 = *((v2df *)&inData[count+8]);
        const v2df in10 = *((v2df *)&inData[count+10]);
        const v2df in12 = *((v2df *)&inData[count+12]);
        const v2df in14 = *((v2df *)&inData[count+14]);
        const v2df in16 = *((v2df *)&inData[count+16]);
        const v2df in18 = *((v2df *)&inData[count+18]);
        const v2df in20 = *((v2df *)&inData[count+20]);
        const v2df in22 = *((v2df *)&inData[count+22]);
        const v2df in24 = *((v2df *)&inData[count+24]);
        const v2df in26 = *((v2df *)&inData[count+26]);
        const v2df in28 = *((v2df *)&inData[count+28]);
        const v2df in30 = *((v2df *)&inData[count+30]);
        accumul += ( in00 * in00 + in02 * in02
                   + in04 * in04 + in06 * in06
                   + in08 * in08 + in10 * in10
                   + in12 * in12 + in14 * in14
                   + in16 * in16 + in18 * in18
                   + in20 * in20 + in22 * in22
                   + in24 * in24 + in26 * in26
                   + in28 * in28 + in30 * in30 );
      }
#pragma omp for reduction(+:temp_sum) nowait
      for(mwIndex count = num32Elems; count < numElems; count++) {
        temp_sum += inData[count] * inData[count];
      }
    }
  }
  const double * const __restrict accumuld = (double *)&accumul;
  return (temp_sum + accumuld[0] + accumuld[1]);
}

inline double
cell_dot_product_sse(const mxArray * const inArray1, const mxArray * const inArray2)
{
  double temp_sum = 0;
  v2df accumul = {0, 0};

  const mwSize numCells = mxGetNumberOfElements(inArray1);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
      const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell1);
      const double * const inData1 = mxGetPr(inCell1);
      const double * const inData2 = mxGetPr(inCell2);

      const mwSize num32Elems = ROUND_DOWN(numElems, 32);

#pragma omp for reduction(+:accumul) nowait
      for(mwIndex count = 0; count < num32Elems; count += 32) {
        const v2df in00 = *((v2df *)&inData1[count+0]) * *((v2df *)&inData2[count+0]);
        const v2df in02 = *((v2df *)&inData1[count+2]) * *((v2df *)&inData2[count+2]);
        const v2df in04 = *((v2df *)&inData1[count+4]) * *((v2df *)&inData2[count+4]);
        const v2df in06 = *((v2df *)&inData1[count+6]) * *((v2df *)&inData2[count+6]);
        const v2df in08 = *((v2df *)&inData1[count+8]) * *((v2df *)&inData2[count+8]);
        const v2df in10 = *((v2df *)&inData1[count+10]) * *((v2df *)&inData2[count+10]);
        const v2df in12 = *((v2df *)&inData1[count+12]) * *((v2df *)&inData2[count+12]);
        const v2df in14 = *((v2df *)&inData1[count+14]) * *((v2df *)&inData2[count+14]);
        const v2df in16 = *((v2df *)&inData1[count+16]) * *((v2df *)&inData2[count+16]);
        const v2df in18 = *((v2df *)&inData1[count+18]) * *((v2df *)&inData2[count+18]);
        const v2df in20 = *((v2df *)&inData1[count+20]) * *((v2df *)&inData2[count+20]);
        const v2df in22 = *((v2df *)&inData1[count+22]) * *((v2df *)&inData2[count+22]);
        const v2df in24 = *((v2df *)&inData1[count+24]) * *((v2df *)&inData2[count+24]);
        const v2df in26 = *((v2df *)&inData1[count+26]) * *((v2df *)&inData2[count+26]);
        const v2df in28 = *((v2df *)&inData1[count+28]) * *((v2df *)&inData2[count+28]);
        const v2df in30 = *((v2df *)&inData1[count+30]) * *((v2df *)&inData2[count+30]);
        accumul += ( in00 + in02 + in04 + in06 + in08 + in10 + in12 + in14
                   + in16 + in18 + in20 + in22 + in24 + in26 + in28 + in30 );
      }
#pragma omp for reduction(+:temp_sum) nowait
      for(mwIndex count = num32Elems; count < numElems; count++) {
        temp_sum += inData1[count] * inData2[count];
      }
    }
  }
  const double * const __restrict accumuld = (double *)&accumul;
  return (temp_sum + accumuld[0] + accumuld[1]);
}
#endif

template<const bool allocate, typename Function>
inline void
cell_iteration(mxArray * & outArray, const mxArray * const inArray,
    const Function & func)
{
  const mwSize numCells = mxGetNumberOfElements(inArray);
  if (allocate) {
    cell_allocate<mxDOUBLE_CLASS>(outArray, inArray);
  }

#pragma omp parallel for
  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * const inCell = mxGetCell(inArray, cellIdx);
    mxArray * const outCell = mxGetCell(outArray, cellIdx);

    const mwSize numElems = mxGetNumberOfElements(inCell);
    const double * const inData = mxGetPr(inCell);
    double * const outData = mxGetPr(outCell);

    for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++) {
      outData[elemIdx] = func(outData[elemIdx], inData[elemIdx]);
    }
  }
}

template<typename Function>
inline void
cell_iteration_copy(mxArray * & outArray, const mxArray * const inArray1,
    const mxArray * const inArray2, const Function & func)
{
  const mwSize numCells = mxGetNumberOfElements(inArray1);
  cell_allocate<mxDOUBLE_CLASS>(outArray, inArray1);

#pragma omp parallel for
  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
    const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
    const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);
    mxArray * const outCell = mxGetCell(outArray, cellIdx);

    const mwSize numElems = mxGetNumberOfElements(inCell1);
    const double * const inData1 = mxGetPr(inCell1);
    const double * const inData2 = mxGetPr(inCell2);
    double * const outData = mxGetPr(outCell);

    for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++) {
      outData[elemIdx] = func(inData1[elemIdx], inData2[elemIdx]);
    }
  }
}


#endif /* INTERNAL_CELL_DEFS_H_ */
