classdef GtCellBenchmarks
    methods (Static)
        function printHeader(funcName, dataSize)
            fprintf('\n%- 40s (Data size: %f GB)\n', funcName, dataSize / 2^30);
        end

        function printResult(message, result, dataVols)
            fprintf([message '\t'], result);
            if (exist('dataVols', 'var'))
                fprintf('Throughput: %f GB/s\n', dataVols / 2^30 / result);
            end
        end

        function arrays = getArrays(numVols, sizeVols)
            arrays = cell(1, numVols);
            for ii = 1:numVols
                arrays{ii} = rand(sizeVols, sizeVols, sizeVols);
            end
        end

        function bytes = getSizeVariable(var)
            info = whos('var');
            bytes = info.bytes;
        end

        function verifyError(reference, computed)
            fprintf('Checking errors in computation:\n')
            fprintf('\t%f\n', max(cellfun(@(x, y)max(max(max(abs(x - y)))), reference, computed)));
        end

        function newVols = benchmarkCopy(vols)
            bytes = GtCellBenchmarks.getSizeVariable(vols);
            %internal_cell_copy.cpp
            GtCellBenchmarks.printHeader('internal_cell_copy', bytes);
            tic();
            newVols = internal_cell_copy(vols);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);
            for ii = 1:length(newVols)
                newVols{ii} = permute(newVols{ii}, [2 1 3]);
            end
        end

        function benchmarkDiv(vols, newVols)
            bytes = 2 * GtCellBenchmarks.getSizeVariable(vols);
            %internal_cell_div_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_div_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_div_assign(dummy2, newVols);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_div_copy(dummy3, newVols);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x, y){x ./ y}, dummy, newVols);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} ./ newVols{ii};
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkProd(vols, newVols)
            bytes = 2 * GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_prod_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_prod_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_prod_assign(dummy2, newVols);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_prod_copy(dummy3, newVols);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x, y){x .* y}, dummy, newVols);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} .* newVols{ii};
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkProdScalar(vols, beta)
            bytes = GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_prod_scalar_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_prod_scalar_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_prod_scalar_assign(dummy2, beta);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_prod_scalar_copy(dummy3, beta);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x){x * beta}, dummy);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} * beta;
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkSumScalar(vols, beta)
            bytes = GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_sum_scalar_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_sum_scalar_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_sum_scalar_assign(dummy2, beta);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_sum_scalar_copy(dummy3, beta);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x){x + beta}, dummy);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} + beta;
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkSum(vols, newVols)
            bytes = 2 * GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_sum_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_sum_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_sum_assign(dummy2, newVols);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_sum_copy(dummy3, newVols);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x, y){x + y}, dummy, newVols);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} + newVols{ii};
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkSub(vols, newVols)
            bytes = 2 * GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_sum_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_sub_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_sub_assign(dummy2, newVols);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_sub_copy(dummy3, newVols);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x, y){x - y}, dummy, newVols);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} - newVols{ii};
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkScaledSum(vols, newVols, beta)
            bytes = 2 * GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_sum_scaled_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_sum_scaled_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_sum_scaled_assign(dummy2, newVols, beta);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_sum_scaled_copy(dummy3, newVols, beta);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x, y){x + y * beta}, dummy, newVols);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} + newVols{ii} * beta;
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkScaledSub(vols, newVols, beta)
            bytes = 2 * GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_sub_scaled_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_sub_scaled_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_sub_scaled_assign(dummy2, newVols, beta);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
            dummy3 = internal_cell_copy(vols);
            tic();
            dummy3 = internal_cell_sub_scaled_copy(dummy3, newVols, beta);
            GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x, y){x - y * beta}, dummy, newVols);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} - newVols{ii} * beta;
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
            GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function benchmarkScaledFISTASum(vols, newVols, beta)
            bytes = 2 * GtCellBenchmarks.getSizeVariable(vols);
            % internal_cell_sum_scaled_assign.cpp
            GtCellBenchmarks.printHeader('internal_cell_sum_FISTA_scaled_[assign, copy]', bytes);
            dummy2 = internal_cell_copy(vols);
            tic();
            dummy2 = internal_cell_sum_FISTA_scaled_assign(dummy2, newVols, beta);
            GtCellBenchmarks.printResult('C (ass):  %f', toc(), bytes);
%             dummy3 = internal_cell_copy(vols);
%             tic();
%             dummy3 = internal_cell_sum_scaled_copy(dummy3, newVols, beta);
%             GtCellBenchmarks.printResult('C (copy): %f', toc(), bytes);

            dummy = vols;
            tic();
            dummy = cellfun(@(x, y){x + (x - y) * beta}, dummy, newVols);
            GtCellBenchmarks.printResult('cellfun:  %f', toc(), bytes);

            dummy = vols;
            tic();
            for ii = 1:length(dummy)
                dummy{ii} = dummy{ii} + (dummy{ii} - newVols{ii}) * beta;
            end
            GtCellBenchmarks.printResult('for loop: %f', toc(), bytes);

            GtCellBenchmarks.verifyError(dummy, dummy2);
%             GtCellBenchmarks.verifyError(dummy, dummy3);
        end

        function allBenchmarks(vols)
            beta = 2.0;

            newVols = GtCellBenchmarks.benchmarkCopy(vols);

            GtCellBenchmarks.benchmarkDiv(vols, newVols);

            GtCellBenchmarks.benchmarkProd(vols, newVols);

            GtCellBenchmarks.benchmarkProdScalar(vols, beta);

            GtCellBenchmarks.benchmarkSum(vols, newVols);

            GtCellBenchmarks.benchmarkSumScalar(vols, beta);

            GtCellBenchmarks.benchmarkSub(vols, newVols);

            GtCellBenchmarks.benchmarkScaledSum(vols, newVols, beta);

            GtCellBenchmarks.benchmarkScaledSub(vols, newVols, beta);

            GtCellBenchmarks.benchmarkScaledFISTASum(vols, newVols, beta);

%             % internal_cell_exponent.cpp
%             fprintf('\ninternal_cell_exponent\n');
%             tic();
%             exponents2 = internal_cell_exponent(vols, 4);
%             toc()
%             tic();
%             exponents = cellfun(@(x){x .^ 4}, vols);
%             toc()
%             cellfun(@(x, y)disp(max(max(max(abs(x - y)))) > eps), exponents, exponents2)
        end
    end
end
