classdef GtPhase < handle
% GTPHASE Is a class that contains all the information of a given phase. All the
% grains are organized in a performance oriented structure and get/set methods
% are provided, to aid the consistent use of the class.
    properties
        grains    = { };       % List of local grain info (see class GtGrain)
        summary   = { };       % List of summary for each grain (see class GtGrainAnalyser)
        conflicts = { };       % List of conflicts for each grain (see class GtConflict)
        hasConflicts   = [];   % Booleans that says if grain has conflict(s)
        nofConflicts   = [];   % Number of conflict(s)
        selectedGrains = [];   % Booleans that enable/disable grains
        completeness = [];     % Index of completeness of a grain [0, 1]
        center       = [];     % Center of mass of the grain
        R_vector     = [];     % Orientation (in Rodrigues space) of the grain
        boundingBox  = [];     % Bounding Box: [x y z lx ly lz] ?
        active     = true;     % Boolean that enables/disables the phase
        phaseName  = '';       % Recognisable name for the phase
        volumeFile = '';       % Path of the mat file which contains the data.
                               % The syntax is: 'filename:field' where field is
                               % the field in the mat file which actually contains
                               % the volume.
        bboxExtremes = [];
        orderedPositions = {};
    end

    methods
        function obj = GtPhase(name, numberOfGrains)
        % GTPHASE/GTPHASE( name, numberOfGrains )
        % Initializes the phase 'name', to the 'numberOfGrains'
            if (~exist('name', 'var'))
                name = '';
            end
            if (~exist('numberOfGrains', 'var'))
                numberOfGrains = 0;
            end
            obj.phaseName = name;
            obj.initGrains(numberOfGrains);
        end

        function initGrains(obj, number)
        % GTPHASE/INITGRAINS( number ) Initializes the phase to the 'number' of
        % grains
            obj.grains    = cell(number,1);
            obj.summary   = cell(number,1);
            obj.conflicts = cell(number,1);
            for ii = 1:number
                obj.grains{ii}    = GtGrain();
                obj.summary{ii}   = GtGrainAnalyser();
                obj.conflicts{ii} = GtConflict();
            end
            obj.hasConflicts     = false(number, 1);
            obj.nofConflicts     = zeros(number, 1);            
            obj.selectedGrains   = true(number, 1);
            obj.completeness     = ones(number, 1);
            obj.center           = zeros(number, 3);
            obj.R_vector         = zeros(number, 3);
            obj.boundingBox      = zeros(number, 6);
            obj.bboxExtremes     = zeros(number, 6);
            obj.orderedPositions = {};
        end

        function pushGrain(obj)
        % GTPHASE/PUSHGRAIN Adds a new empty grain at the end of the list
            obj.grains{end+1}    = GtGrain();
            obj.summary{end+1}   = GtGrainAnalyser();
            obj.conflicts{end+1} = GtConflict();
            obj.hasConflicts     = [obj.hasConflicts; false];
            obj.nofConflicts     = [obj.nofConflicts; 0];            
            obj.selectedGrains   = [obj.selectedGrains; true];
            obj.completeness     = [obj.completeness; 1];
            obj.center           = [obj.center; 0 0 0];
            obj.R_vector         = [obj.R_vector; 0 0 0];
            obj.boundingBox      = [obj.boundingBox; 0 0 0 0 0 0];
            obj.bboxExtremes     = [obj.bboxExtremes; 0 0 0 0 0 0];
            obj.orderedPositions = {};
        end

                
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Setter Functions

        % phase fields
        function setHasConflicts(obj, grainid, hasConf)
            obj.hasConflicts(grainid, 1) = hasConf;
        end
        
        function setNofConflicts(obj, grainid, nofConf)
            obj.nofConflicts(grainid, 1) = nofConf;
        end

        function setSelected(obj, grainid, sel)
            obj.selectedGrains(grainid, 1) = sel;
        end

        function setCompleteness(obj, grainid, compl)
            if ((compl < 0) || (compl > 1))
                error('DATA:wrong_parameter', ...
                      ['Completeness should be between [0, 1], but is: ' num2str(compl)]);
            end
            obj.completeness(grainid, 1) = compl;
        end

        function setCenter(obj, grainid, center)
            obj.center(grainid, :) = center;
        end

        function setR_vector(obj, grainid, R_vec)
            obj.R_vector(grainid, :) = R_vec;
        end

        function setBoundingBox(obj, grainid, bbox)
            obj.boundingBox(grainid, :) = bbox;
            obj.bboxExtremes(grainid, :) = [bbox(1:3) (bbox(1:3) + bbox(4:6) -1) ];
        end

        % grains fields
        function setThreshold(obj, grainid, thr)
            obj.grains{grainid}.threshold = thr;
        end

        function setSelectedDiffspots(obj, grainid, selDiffspots)
        % function setSelectedDiffspots(obj, grainid, selDiffspots)
            obj.grains{grainid}.selectedDiffspots = selDiffspots;
        end

        function setSelectedDiffspotsIndexes(obj, grainid, indexes, value)
        % function setSelectedDiffspotsIndexes(obj, grainid, indexes, value)
            obj.grains{grainid}.selectedDiffspots(indexes) = value;
        end

        % summary fields
        function setTotSpots(obj, grainid, tot)
            obj.summary{grainid}.totspots = tot;
        end

        function setIdSpot(obj, grainid, id)
            obj.summary{grainid}.idspot = id;
        end

        function setIdSpotIndexes(obj, grainid, ind)
            obj.summary{grainid}.idspot(2,:) = ind;
        end

        function setIdSpotIds(obj, grainid, ids)
            obj.summary{grainid}.idspot(1,:) = ids;
        end

        function setNumGrain(obj, grainid, num)
            obj.summary{grainid}.numgrain = num;
        end

        function setIdConflict(obj, grainid, id)
            obj.summary{grainid}.idconflict = id;
        end 

        function setDelete(obj, grainid, del)
            obj.summary{grainid}.delete = del;
        end

        function setFlag(obj, grainid, f)
            obj.summary{grainid}.flag = f;
        end

        function setDifspotID(obj, grainid, spots)
            obj.summary{grainid}.difspotID = spots;
        end

        % conflicts fields
        function setDifspots(obj, grainid, conflictid, spots)
            obj.conflicts{grainid}(conflictid).difspots = spots;
        end

        function setMaxNum(obj, grainid, conflictid, max)
            obj.conflicts{grainid}(conflictid).maxnum = max;
        end

        function setMerge(obj, grainid, conflictid, m)
            obj.conflicts{grainid}(conflictid).merge = m;
        end

        function setAllEquals(obj, grainid, conflictid, alleq)
            obj.conflicts{grainid}(conflictid).allequals = alleq;
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Getter Functions

        % phase fields
        function hasC = getHasConflicts(obj, grainid)
            hasC = obj.hasConflicts(grainid, 1);
        end
      
        function nofC = getNofConflicts(obj, grainid)
            nofC = obj.nofConflicts(grainid, 1);
        end

        function sel = getSelected(obj, grainid)
            sel = obj.selectedGrains(grainid, 1);
        end

        function compl = getCompleteness(obj, grainid)
            compl = obj.completeness(grainid, 1);
        end

        function center = getCenter(obj, grainid)
            center = obj.center(grainid, :);
        end

        function center = getCenterPixels(obj, grainid, pixelsize)
            center = obj.center(grainid, :)/pixelsize;
        end

        function R_vec = getR_vector(obj, grainid)
            R_vec = obj.R_vector(grainid, :);
        end

        function bbox = getBoundingBox(obj, grainid)
            bbox = obj.boundingBox(grainid, :);
        end

        function bbox = getBoundingBoxExtremes(obj, grainid)
            bbox = obj.bboxExtremes(grainid, :);
        end

        function shift = getBoundingBoxShift(obj, grainid)
            shift = obj.boundingBox(grainid, 1:3);
        end

        % grains fields
        function thr = getThreshold(obj, grainid)
            thr = obj.grains{grainid}.threshold;
        end

        function selDiffspots = getSelectedDiffspots(obj, grainid)
            selDiffspots = obj.grains{grainid}.selectedDiffspots;
        end

        function grain = getGrain(obj, grainid)
            grain = obj.grains{grainid};
        end

        % summary fields
        function tot = getTotSpots(obj, grainid)
            tot = obj.summary{grainid}.totspots;
        end

        function idspot = getIdSpot(obj, grainid)
            idspot = obj.summary{grainid}.idspot;
        end

        function indexes = getIdSpotIndexes(obj, grainid)
            indexes = obj.summary{grainid}.idspot(2,:);
        end

        function ids = getIdSpotIds(obj, grainid)
            ids = obj.summary{grainid}.idspot(1,:);
        end

        function num = getNumGrain(obj, grainid)
            num = obj.summary{grainid}.numgrain;
        end
        
        function id = getIdConflict(obj, grainid)
            id = obj.summary{grainid}.idconflict;
        end
        
        function del = getDelete(obj, grainid)
            del = obj.summary{grainid}.delete;
        end
        
        function f = getFlag(obj, grainid)
            f = obj.summary{grainid}.flag;
        end
        
        function ids = getDifspotID(obj, grainid)
            ids = obj.summary{grainid}.difspotID;
        end

        function summary = getSummary(obj, grainid)
            summary = obj.summary{grainid};
        end

        % conflicts fields
        function spots = getDifspots(obj, grainid, conflictid)
            spots = obj.conflicts{grainid}(conflictid).difspots;
        end

        function max = getMaxNum(obj, grainid, conflictid)
            max = obj.conflicts{grainid}(conflictid).maxnum;
        end

        function m = getMerge(obj, grainid, conflictid)
            m = obj.conflicts{grainid}(conflictid).merge;
        end

        function alleq = getAllEquals(obj, grainid, conflictid)
            alleq = obj.conflicts{grainid}(conflictid).allequals;
        end

        function conflicts = getConflicts(obj, grainid, conflictid)
            conflicts = obj.conflicts{grainid}(conflictid);
        end

        % get all values from grains
        function allvalues = getAllValuesFromGrainsField(obj, field, uniform)
            grainsArray = [obj.grains{:}];
            allvalues = cellfun(@(name) name, {grainsArray.(field)}, 'UniformOutput', uniform);
        end

        % get all values from summary
        function allvalues = getAllValuesFromSummaryField(obj, field, uniform)
            summaryArray = [obj.summary{:}];
            allvalues = cellfun(@(name) name, {summaryArray.(field)}, 'UniformOutput', uniform);
        end

        % get all values from conflicts
        function [allvalues, grainid] = getAllValuesFromCellstrField(obj, cellstr, field)
        %     [allvalues, grainid] = getAllValuesFromCellstrField(obj, cellstr, field)
        %     ------------------------------------------------------------------------
        %     cellstr = 'grains'/'summary'/'conflicts'

            cellArray = obj.(cellstr);
            if ~strcmpi(cellstr, 'conflicts')
                cellArray = [cellArray{:}];
            end
            allvalues = [];
            grainid   = [];
            count = 0;
            sizeA = [];
            for i = 1 : size(cellArray,1)
                array = cellArray{i};
                sizeA = [sizeA, size(array,1)];
                for j = 1 : size(array,1)
                    current = array(j);
                    if isempty(current.(field))
                        count = count + 1;
                    else
                        grainid = [grainid; i];
                        allvalues = [allvalues; current.(field)];
                    end
                    fprintf('grainid: %04d size: %2d currentID: %2d isempty: %4d total: %4d length: %4d \n', ...
                        i, size(array,1) , j, count, size(allvalues, 1), size(grainid,1))
                end
            end
            sizeA = cumsum(sizeA);
            disp(['empty: ' num2str(count)])
        end

        % less than
        function grainIDs = getGrainsWithFieldValueLessThan(obj, field, value)
            allvalues = getAllValuesFromGrainsField(obj, field, true);
            grainIDs = find(allvalues < value);
        end
        
        function grainIDs = getGrainsWithFieldValueEqualsTo(obj, field, value)
            allvalues = getAllValuesFromGrainsField(obj, field, true);
            grainIDs = find(allvalues == value); 
        end

        function grainIDs = getSummaryWithFieldValueLessThan(obj, field, value, uniform)
            allvalues = getAllValuesFromSummaryField(obj, field, uniform);
            ids = find(obj.hasConflicts == true);
            allvalues = [allvalues{:}];
            grainIDs = ids(allvalues < value);
        end

        function grainIDs = getSummaryWithFieldValueMoreThan(obj, field, value, uniform)
            allvalues = getAllValuesFromSummaryField(obj, field, uniform);
            ids = find(obj.hasConflicts == true);
            allvalues = [allvalues{:}];
            grainIDs = ids(allvalues > value);
        end

        function grainIDs = getSummaryWithFieldValueEqualsTo(obj, field, value, uniform)
            allvalues = getAllValuesFromSummaryField(obj, field, uniform);
            ids = find(obj.hasConflicts == true);
            allvalues = [allvalues{:}];
            grainIDs = ids(allvalues == value);
        end

        function grainIDs = getConflictsWithFieldValueEqualsTo(obj, field, value)
            [allvalues, ids] = getAllValuesFromConflictsField(obj, field);
            indexes = allvalues == value;
            grainIDs = ids(indexes); 
        end

        % between 0 and value
        function grainIDs = getGrainsWithNofConflictsLessThan(obj, value)
            grainIDs = find(obj.nofConflicts < value & obj.hasConflicts == true);
        end

        function grainIDs = getGrainsWithCompletenessLessThan(obj, value)
            grainIDs = find(obj.completeness < value & obj.completeness > 0);
        end

        function grainIDs = getGrainsWithDiffspotsNumLessThan(obj, value)
            grainsArray = [obj.grains{:}];
            numDiffSpots = cellfun(@length, {grainsArray.selectedDiffspots});
            grainIDs = find(numDiffSpots < value);
        end

        function numDiffSpots = getSelectedDiffspotsNum(obj, grainid)
            diffSpots = find(obj.grains{grainid}.selectedDiffspots == true);
            numDiffSpots = length(diffSpots);
        end

        function spotIDs = getSelectedDiffspotsTrue(obj, grainid)
            spotIDs = find(obj.grains{grainid}.selectedDiffspots == true);
        end

        % more than value
        function grainIDs = getGrainsWithNofConflictsMoreThan(obj, value)
            grainIDs = find(obj.nofConflicts > value & obj.hasConflicts == true);
        end

        % logical values
        function inconflict = getHasConflictsTrue(obj)
            inconflict = find(obj.hasConflicts == true);
        end

        function deselected = getDeselectedGrains(obj)
            deselected = find(obj.selectedGrains == false);
        end
        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Analysis Functions
        function buildBBSearchList(obj)
            for ii = 1:size(obj.bboxExtremes, 2)
                [obj.orderedPositions{ii,1:2}] = sort(obj.bboxExtremes(:,ii));
            end
        end

        function matchList = simpleSearchBBtouched(obj, point)
            matchList = find(  obj.bboxExtremes(:,1) <= point(1) ...
                             & obj.bboxExtremes(:,2) <= point(2) ...
                             & obj.bboxExtremes(:,3) <= point(3) ...
                             ...
                             & obj.bboxExtremes(:,4) >= point(1) ...
                             & obj.bboxExtremes(:,5) >= point(2) ...
                             & obj.bboxExtremes(:,6) >= point(3) ...
                            );
        end

        function IDs = getNeighbours(obj, grainID, oversize)
            if (~exist('oversize', 'var'))
                oversize = 1;
            end
            numGrains = length(obj.grains);
            replOnes = ones(numGrains, 1);

            halfLengths = obj.boundingBox(:, 4:6) / 2;
            grainHalfLengths = halfLengths(grainID, :);

            maxDistances = halfLengths + grainHalfLengths(replOnes, :);

            centersInPixels = obj.boundingBox(:, 1:3) + halfLengths;
            grainCenter = centersInPixels(grainID, :);
            relativeCenters = abs(centersInPixels - grainCenter(replOnes, :));

            IDs = setdiff(find(all(relativeCenters < (maxDistances * oversize), 2)), grainID);
        end
        
        function idspot = buildIdSpot(obj, grainid)
            difspotID = obj.summary{grainid}.difspotID'; % row vector
            idspot = obj.summary{grainid}.idspot; % row vector
            if ~isempty(idspot)
                if size(idspot,1) < size(idspot,2)
                    idspot = idspot(1,:);
                else
                    idspot = idspot(:,1);
                end
                if size(difspotID,1) ~= size(idspot,1)
                    idspot = idspot';
                end
                [spots, ind] = intersect(difspotID, idspot);              
                IDspot(1,:) = spots;
                IDspot(2,:) = ind;
                obj.summary{grainid}.idspot = IDspot;
            else
                spots = [];
                obj.summary{grainid}.idspot = [];
            end
            idspot = spots;
        end
        
        function merge = checkOverlapConflict(obj, id1, id2, pixelsize)
        % function merge = checkOverlapConflict(obj, id1, id2, pixelsize)
            merge_value1 = false;
            merge_value2 = false;

            % centers
            c1 = obj.center(id1,:) / pixelsize;
            c2 = obj.center(id2,:) / pixelsize;
            dist = norm(c1-c2);
            b1 = obj.boundingBox(id1, :);
            b2 = obj.boundingBox(id2, :);
            % mean of two volumes : get radius equivalent and get 10% of it
            % as threshold
            thr_dist = mean(prod(b2(4:6)), prod(b1(4:6))) ^ (1/3) / 10;
            %thr_dist = 0.10 * (0.6/2)/pixelsize; % 10% of radius of sample
            if dist < thr_dist
                merge_value1 = true;
            end
            
            % R_vector
            r1 = obj.R_vector(id1,:);
            r2 = obj.R_vector(id2,:);
            %fprintf('R_vector1: %5.4f %5.4f %5.4f\nR_vector2: %5.4f %5.4f %5.4f\n',r1,r2)
            angle = atan2(norm(cross(r1,r2)), dot(r1,r2));
            angle = rad2deg(angle);
            %disp(['angle: ' num2str(angle) ' deg'])
            %disp('thr_angle = 1 deg')

            if angle < 15
                merge_value2 = true;
            end
            
            merge.id1 = id1;
            merge.id2 = id2;
            merge.c1 = c1;
            merge.c2 = c2;
            merge.dist = dist;
            merge.b1 = b1;
            merge.b2 = b2;
            merge.value1 = merge_value1;

            merge.r1 = r1;
            merge.r2 = r2;
            merge.angle = angle;
            merge.value2 = merge_value2;
            
            merge.value = merge_value1 & merge_value2;
        end
        
    end
end
