/**
 * Original file by Christian Nemoz from ID17 @ ESRF nemoz@esrf.eu
 * Cleaned and optimized by Nicola Vigano from ID11 @ ESRF vigano@esrf.eu
 */
#include <math.h>
#include "mex.h"

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  /* Incoming image dimensions */
  const unsigned int num_cols = mxGetM(prhs[0]), num_rows = mxGetN(prhs[0]);

  /* Pointers to incoming matrices: */
  const double * const image_in = mxGetPr(prhs[0]);
  /* Shift matrices */
  const double * const shiftx   = mxGetPr(prhs[1]);
  const double * const shifty   = mxGetPr(prhs[2]);

  /* Create a matrix for the return argument */
  plhs[0] = mxCreateDoubleMatrix(num_cols, num_rows, mxREAL);

  {
    /* Pointers to outgoing matrix: */
    double * const image_out = mxGetPr(plhs[0]);

    unsigned int pt = 0;
    const double * image_in_tmp = image_in;
    unsigned int j = 0;
    for(j = 0; j < num_cols; j++)
    {
      unsigned int i = 0;
      for(i = 0; i < num_rows; i++, pt++, image_in_tmp++)
      {
        /* compute 4 points defining target pixel, a rectangle. Corners are
         * defined by mean values of shifts taken 2 by 2 */
        /* FIXME SLOW pointer arithmetics */
        const unsigned int pt1  = pt + ( i < ( num_cols - 1 ) );
        const unsigned int ptn  = pt + num_cols*( j < (num_rows-1) );
        const unsigned int ptn1 = ptn + ( i < ( num_cols - 1 ) );

        /* target pixel coordinates: the pixel where the original pixel needs to
         * be shifted */
        const double x1 = i   + 0.5*(shiftx[pt ]+shiftx[ptn ]);
        const double x2 = i+1 + 0.5*(shiftx[pt1]+shiftx[ptn1]);
        const double y1 = j   + 0.5*(shifty[pt ]+shifty[pt1 ]);
        const double y2 = j+1 + 0.5*(shifty[ptn]+shifty[ptn1]);

        /* target coordinates within image? */
        const int is_ok =  ( x1 >= 0. ) && ( x1 < num_cols )  &&
                          (( y1 >= 0. ) && ( y1 < num_rows )) &&
                           ( x2 >= 0. ) && ( x2 < num_cols )  &&
                          (( y2 >= 0. ) && ( y2 < num_rows ));

        if (is_ok)
        {
          /* compute surface of target pixel */
          const double val_sur_surf = (*image_in_tmp) / fabs( (x2-x1)*(y2-y1) );

          /* assume x2>x1 and y2>y1, and round values! */
          const unsigned int x_lower_lim_target = x1;
          const unsigned int x_upper_lim_target = x2;
          const unsigned int y_lower_lim_target = y1;
          const unsigned int y_upper_lim_target = y2;

          /* get the rounding */
          const double i11_x1 = (x_lower_lim_target+1 -                 x1);
          const double x2_i2  = (x2                   - x_upper_lim_target);
          const double x2_x1  = (x2                   -                 x1);
          const double j11_y1 = (y_lower_lim_target+1 -                 y1);
          const double y2_j2  = (y2                   - y_upper_lim_target);
          const double y2_y1  = (y2                   -                 y1);

          register unsigned int ky;
          for(ky = y_lower_lim_target; ky <= y_upper_lim_target; ky++)
          {
            const double y = (ky == y_lower_lim_target)
                ? ((y_upper_lim_target==y_lower_lim_target) ? y2_y1 : j11_y1)
                : ((ky==y_upper_lim_target)                 ? y2_j2 : 1.);

            register unsigned int kx;
            for(kx = x_lower_lim_target; kx <= x_upper_lim_target; kx++)
            {
              const double x = (kx == x_lower_lim_target)
                  ? ((x_upper_lim_target==x_lower_lim_target) ? x2_x1 : i11_x1)
                  : ((kx==x_upper_lim_target)                 ? x2_i2 : 1.);

              /* shifts the values to the target pixels */
              image_out[kx+ ky*num_cols] += (x*y*val_sur_surf);
            }
          }
        } /* end is_ok */
      } /* end i */
    } /* end j */
  }
}
