function [img, varargout] = edf_read(filename, GT_bb, dispmes, info)
  % EDF_READ.M Reads images and 3D volumes from .edf files
  % Usage:
  % data = edf_read(filename[, GT_bb]);
  % [data, info] = edf_read(filename[, GT_bb]);
  %
  % if GT_bb is supplied (optional) - only reads the region from [originx
  % originy width height] inclusive (only works for 2D EDF files)
  %
  % if a second output variable is given, returns the edf header info
  %
  % Greg Johnson
  % June 2006
  %
  % Modified by Nicola Vigano, 2011-2012, nicola.vigano@esrf.fr

  if (~exist('dispmes', 'var'))
    dispmes = true;
  else
    dispmes = ~strcmp(dispmes, 'nodisp');
  end

  %% Acquire file information
  edfFile = dir(filename);

  %% File Error Checking
  if (isempty(edfFile))
    mexc = MException('EDF:fileNotFound', ['Could not find ' filename]);
    throw(mexc);
  end

  if (edfFile.isdir == true)
    mexc = MException('EDF:fileIsDirectory', [ filename ' is a directory instead of a file.']);
    throw(mexc);
  end

  if (edfFile.bytes == 0)
    mexc = MException('EDF:fileIsEmpty', [ filename ' is an empty file.']);
    throw(mexc);
  end

  % Let's read the header
  if (~exist('info', 'var') || isempty(info))
    info = edf_info(filename);
  elseif (~isfield(info, 'headerlength') || info.headerlength < 0)
    % Make it sure the header length is found/computed!
    info.headerlength = findHeaderLength(filename, info.byteorder);
  end

  % Test and fix third dimension
  if (~isfield(info,'dim_3'))
    info.dim_3 = 1;
  else
    if info.dim_3 > 1 && (exist('GT_bb','var') && ~isempty(GT_bb))
      mexc = MException('EDF:roiIn3d', 'Cannot use an ROI on 3D EDF file');
      throw(mexc);
    end
  end

  fid = fopen(filename,'r',info.byteorder);
  if (fid == -1)
    mexc = MException('EDF:fileNotFound', ['Could not open ' fname]);
    throw(mexc);
  end
  fseek(fid, info.headerlength, 'bof');

  %%%%%%%%%%%%
  %ROI

  if exist('GT_bb','var') && ~isempty(GT_bb)
    % GT_bb = [originx originy width height]
    kodak_midline=1024-GT_bb(2)+1;

    switch info.datatype
      case {'uint8', 'int8'}
        nbytes = 1;
      case {'uint16', 'int16'}
        nbytes = 2;
      case {'uint32', 'int32', 'float32'}
        nbytes = 4;
      case {'uint64', 'int64', 'float64'}
        nbytes = 8;
    end

    % ROI was specified
    pr_str = sprintf('%d*%s', GT_bb(3), info.datatype);
    skip = nbytes * (info.dim_1 - GT_bb(3));
    fseek(fid, nbytes*(info.dim_1*(GT_bb(2)-1)+GT_bb(1)-1), 0);
    [img, count] = fread(fid, [GT_bb(3),GT_bb(4)], pr_str, skip);
    dim_1 = GT_bb(3);
    dim_2 = GT_bb(4);
  else
    %%%%%%%%%%%%%
    kodak_midline = 1024;
    dim_1 = info.dim_1;
    dim_2 = info.dim_2;
    [img, count] = fread(fid, info.dim_1*info.dim_2*info.dim_3, info.datatype);
  end
  fclose(fid);

  %% Check if the file was corrupted
  if count ~= (dim_1*dim_2*info.dim_3)
    report_string = [ 'File ' filename ' contains less data:\n' ...
                      '  Expected: ' num2str(dim_1*dim_2*info.dim_3) ...
                      ', Got: ' num2str(count)];
    mexc = MException('EDF:fileCorrupted', report_string );
    throw(mexc);
  end

  %% Let's reshape to the real format of the matrix
  img = reshape(img, dim_1, dim_2, info.dim_3);

  if info.dim_3 > 1 % 3D volume
    img = permute(img, [2 1 3]); % reorganise dimensions
  else % 2D image
    img = transpose(img);
  end

  % Kodak4MV1: do any special processing required
  if isfield(info,'ccdmodel') && any(isfield(info,{'isccdcorrected', 'israw'}))
      % long test to keep compatible with 'israw' and 'isccdcorrected'
      % fields.  Should be able to remove 'israw' in due course
    if strcmpi(info.ccdmodel,'kodak4mv1') ...
       && ( (isfield(info, 'isccdcorrected') && strcmpi(info.isccdcorrected,'false')) ...
            || (isfield(info, 'israw') && strcmpi(info.israw,'true')))
      if kodak_midline > 0 && info.dim_2 == 2048
        if dispmes == true
          disp('Adding missing row for Kodak4M sensor and applying 0.96 gamma')
        end
        imc = [ img(1:kodak_midline,:); ...
                mean([img(kodak_midline,:); ...
                img(kodak_midline+1,:)]); ...
                img(kodak_midline+1:end-1,:) ];
        img = imc .^ 0.96;
      else
        if (dispmes == true)
          disp('WARNING: Cropped image may be corrupted due to Kodak4M sensor.')
        end
      end

    end
  end

  %% If requested, let's output the header
  if (nargout == 2)
    varargout{1} = info;
  end

    function headerLength = findHeaderLength(filename, byteorder)
        fidFind = fopen(filename, 'r', byteorder);
        if (fidFind == -1)
            mexcf = MException('EDF:fileNotFound', ['Could not open ' fname]);
            throw(mexcf);
        end

        for ii = 1:8
            fseek(fidFind, 512*ii -2, 'bof');
            if (fgets(fidFind, 1) == '}')
                fclose(fidFind);
                headerLength = 512*ii;
                return
            end
        end

        fprintf('Malformed header, searching manually.\n');
        fseek(fidFind, 1, 'bof');
        endofheader = false;
        while (~endofheader)
            htxt = fgetl(fidFind);
            endofheader = find( htxt == '}' );
        end
        headerLength = ftell(fidFind);
        fclose(fidFind);
    end
end %end of function
