/* jacobs_ray_3d
% void jacobs_ray_3d(int im_size, double *start, double *end, int *ray_index, double *ray_data, int *n_entries)
%
% implementation of algorithm by Jacobs et.al (modification of Siddon's)
%
% int im_size: size of grid
% double start[3]: x,y,z coordinates for starting point of ray
% double end[3]
% int ray_index[n_entries]: stores voxel numbers hit by ray
% double ray_data[n_entries]: stores weights corresponding to relative length of ray in a given voxel
% int n_entries: counts number of voxels hit by ray
%
%
% takes coordinates of points relative origin in centre of grid, and
% translates to correspond to algorithm
%
%
% david szotten
% august 2005
*/

#include <math.h>
#include <matrix.h>
#include <stdlib.h>
#include <stdio.h>
#include "reconstruct_phototom.h"


/*#ifndef MIN3X
	#define MIN3X(x,y,z) ((x) < (y) ? MINX((x),(z)) : MINX((y),(z)) )
	#define MAX3X(x,y,z) ((x) > (y) ? MAXX((x),(z)) : MAXX((y),(z)) )
#endif*/


int equal_to_precision(double x, double y, double precision);
double alpha_fn(int n, double p1, double p2, double b, double d);
double p(double alpha, double p1, double p2);
double phi(double alpha, double p1, double p2, double b, double d);

double min_dbl(double a, double b) { return a < b ? a : b; };
double max_dbl(double a, double b) { return a > b ? a : b; };

double min3_dbl(double a, double b, double c) { return a < b ? min_dbl(a,c) : min_dbl(b,c); };
double max3_dbl(double a, double b, double c) { return a > b ? max_dbl(a,c) : max_dbl(b,c); };



void jacobs_ray_3d(int im_size, double *start, double *end, int *ray_index, double *ray_data, int *n_entries,
		   double voxel_size)
{

  int testing = 0;
/*  if((*n_entries)==830530) {
	  printf(":jacobs_ray_3d");fflush(NULL);
	  testing = 0*1;
  }
*/

	int N_x, N_y, N_z, N_p;
	double b_x, b_y, b_z, d_x, d_y, d_z, d_conv; /*, d_conv_alpha; */
	double p1_x, p1_y, p1_z, p2_x, p2_y, p2_z;

	int i,j,k;

	double alpha_x_min, alpha_y_min, alpha_z_min, alpha_x_max, alpha_y_max, 
alpha_z_max, alpha_min, alpha_max, alpha_x, alpha_y, alpha_z, alpha_c;
	double alpha_x_u, alpha_y_u, alpha_z_u;
	double l_ij;
	int i_min, j_min, k_min, i_max, j_max, k_max, n_count, i_u, j_u, k_u;

	*n_entries=0;
	
	p1_x = start[0];
	p1_y = start[1];
	p1_z = start[2];
	p2_x = end[0];
	p2_y = end[1];
	p2_z = end[2];

	b_x=-im_size*voxel_size/2;
	b_y=-im_size*voxel_size/2;
	b_z=-im_size*voxel_size/2;

	d_x=voxel_size;
	d_y=voxel_size;
	d_z=voxel_size;

	/* use total lengh=alpha_max-alpha_min instead, to get everage, not sum. */
	/* moving back to original d_conv*/
	d_conv=sqrt( (p1_x-p2_x)*(p1_x-p2_x) + (p1_y-p2_y)*(p1_y-p2_y) +
(p1_z-p2_z)*(p1_z-p2_z));/**/

	N_x=im_size+1;
	N_y=im_size+1;
	N_z=im_size+1;

	/* rewrite in terms of jacob2d*/
	if (equal_to_precision(p1_x,p2_x,PRECISION)==1) {
    		i=(int) floor( phi(0, p1_x, p2_x, b_x, d_x));
    		/* or ceil? */

    		/*may as well assume ray starts and ends outside grid as this will be
    		the case whenever this is used */
		if (0 <= i && i <= im_size-1) {

			int *ray2d_index_j;
			int *ray2d_index_k;
			double *ray2d_data;
			int ray2d_n_entries;
			
			/*ray2d_index_j=mxCalloc( im_size*3, sizeof(int));
			ray2d_index_k=mxCalloc( im_size*3, sizeof(int));
			ray2d_data=mxCalloc( im_size*3,sizeof(double));/**/
			
			ray2d_index_j=malloc( im_size*3* sizeof(int));
			ray2d_index_k=malloc( im_size*3* sizeof(int));			
			ray2d_data=calloc( im_size*3, sizeof(double));/**/
			if(testing == 1)
			  printf(".");fflush(NULL);
			
			if (ray2d_index_j == NULL || ray2d_index_k == NULL || ray2d_data == NULL ) {
				/*mexWarnMsgTxt("Not enough memory!");*/
				printf("Error");fflush(NULL);/**/
				return;
			}

			if(testing == 1)
			  printf("running 2d...");fflush(NULL);

			jacobs_ray_2d(im_size, p1_y, p1_z, p2_y, p2_z, ray2d_index_j, ray2d_index_k,
				      ray2d_data, &ray2d_n_entries, voxel_size);
			
			if(testing == 1)
			  printf("putting in place...");fflush(NULL);
			
			/* loop through entries in ray */
			while (ray2d_n_entries > 0) {
			  ray_index[(*n_entries)] = ray2d_index_k[ray2d_n_entries-1]*im_size*im_size +
			    ray2d_index_j[ray2d_n_entries-1]*im_size + i;
			  ray_data[(*n_entries)++] = ray2d_data[(ray2d_n_entries--) -1];
			}
			
			if(ray2d_index_j != NULL)
			  /*mxFree(ray2d_index_j);/**/
			  free(ray2d_index_j);/**/
			if(ray2d_index_k != NULL)
			  /*mxFree(ray2d_index_k);/**/
			  free(ray2d_index_k);/**/
			if(ray2d_data != NULL)
			  /*mxFree(ray2d_data);/**/
			  free(ray2d_data);/**/
			
		}
		/* else  leave empty */

		/*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/

		if(testing == 1)
		  printf("returns here (3)");fflush(NULL);		
		return;
	}

	else if (equal_to_precision(p1_y,p2_y,PRECISION)==1) {
		j=(int) floor( phi(0, p1_y, p2_y, b_y, d_y));
		/* or ceil? */

		/*may as well assume ray starts and ends outside grid as this will be
    		%the case whenever this is used */
		if (0 <= j && j <= im_size-1) {

			int *ray2d_index_i;
			int *ray2d_index_k;
			double *ray2d_data;
			int ray2d_n_entries;

/*			ray2d_index_i=mxCalloc( im_size*3, sizeof(int));
			ray2d_index_k=mxCalloc( im_size*3, sizeof(int));
			ray2d_data=mxCalloc( im_size*3,sizeof(double));*/

			ray2d_index_i=calloc( im_size*3, sizeof(int));
			ray2d_index_k=calloc( im_size*3, sizeof(int));
			ray2d_data=calloc( im_size*3,sizeof(double));


			if (ray2d_index_i == NULL || ray2d_index_k == NULL || ray2d_data == NULL ) {
				/*mexWarnMsgTxt("Not enough memory!");/**/
				printf("Error");
				return;
			}

			jacobs_ray_2d(im_size, p1_x, p1_z, p2_x, p2_z, ray2d_index_i, ray2d_index_k,
				      ray2d_data, &ray2d_n_entries, voxel_size);

			/* loop through entries in ray */
			while (ray2d_n_entries > 0) {
				ray_index[(*n_entries)] = ray2d_index_k[ray2d_n_entries-1]*im_size*im_size + 
				  j*im_size + ray2d_index_i[ray2d_n_entries-1];
				ray_data[(*n_entries)++] = ray2d_data[(ray2d_n_entries--)-1];
			}

			if(ray2d_index_i != NULL)
			  free(ray2d_index_i);
			if(ray2d_index_k != NULL)
			  free(ray2d_index_k);
			if(ray2d_data != NULL)
			  free(ray2d_data);
		}

		/* else  leave empty */


		/*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/
		if(testing == 1)
		  printf("returns here (4)");fflush(NULL);
		return;
	}

	else if (equal_to_precision(p1_z,p2_z,PRECISION)==1) {
		k=(int) floor( phi(0, p1_z, p2_z, b_z, d_z));
		
		/* or ceil? */

		/*may as well assume ray starts and ends outside grid as this will be
	    	%the case whenever this is used */
		if (0 <= k && k <= im_size-1) {

			int *ray2d_index_i;
			int *ray2d_index_j;
			double *ray2d_data;
			int ray2d_n_entries;

			ray2d_index_i=calloc( im_size*3, sizeof(int));
			ray2d_index_j=calloc( im_size*3, sizeof(int));
			ray2d_data=calloc( im_size*3,sizeof(double));

			if (ray2d_index_i == NULL || ray2d_index_j == NULL || ray2d_data == NULL ) {
				/*mexWarnMsgTxt("Not enough memory!");*/
				printf("Error");
				return;
			}

			jacobs_ray_2d(im_size, p1_x, p1_y, p2_x, p2_y, ray2d_index_i, ray2d_index_j, 
				      ray2d_data, &ray2d_n_entries, voxel_size);

			/* loop through entries in ray */
			while (ray2d_n_entries > 0) {
				ray_index[(*n_entries)] = k*im_size*im_size + 
ray2d_index_j[ray2d_n_entries-1]*im_size + ray2d_index_i[ray2d_n_entries-1];
				ray_data[(*n_entries)++] = ray2d_data[(ray2d_n_entries--) -1];
			}

			if(ray2d_index_i != NULL)
			  free(ray2d_index_i);
			if(ray2d_index_j != NULL)
			  free(ray2d_index_j);
			if(ray2d_data != NULL)
			  free(ray2d_data);		
}
		
		/* else leave empty */

		/*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/
		if(testing == 1)
		  printf("returns here");fflush(NULL);

		return;
	}

	/* x,y,z all different 1-2 */
	else {
	/*mexWarnMsgTxt("Not using 2d");/**/

		alpha_x_min=min_dbl(alpha_fn(0, p1_x, p2_x, b_x, d_x), alpha_fn(N_x-1, p1_x, 
p2_x, b_x, d_x));
    		alpha_x_max=max_dbl(alpha_fn(0, p1_x, p2_x, b_x, d_x), alpha_fn(N_x-1, p1_x, 
p2_x, b_x, d_x));

    		alpha_y_min=min_dbl(alpha_fn(0, p1_y, p2_y, b_y, d_y), alpha_fn(N_y-1, p1_y, 
p2_y, b_y, d_y));
    		alpha_y_max=max_dbl(alpha_fn(0, p1_y, p2_y, b_y, d_y), alpha_fn(N_y-1, p1_y, 
p2_y, b_y, d_y));

    		alpha_z_min=min_dbl(alpha_fn(0, p1_z, p2_z, b_z, d_z), alpha_fn(N_z-1, p1_z, 
p2_z, b_z, d_z));
    		alpha_z_max=max_dbl(alpha_fn(0, p1_z, p2_z, b_z, d_z), alpha_fn(N_z-1, p1_z, 
p2_z, b_z, d_z));

    		alpha_min=max3_dbl(alpha_x_min, alpha_y_min, alpha_z_min);
    		alpha_max=min3_dbl(alpha_x_max, alpha_y_max, alpha_z_max);

		if(testing == 1)
		  printf("get here");fflush(NULL);

		if (alpha_min < alpha_max) {

			/* use total lengh=alpha_max-alpha_min instead, to get everage, not sum. */
			/* d_conv_alpha=alpha_max-alpha_min;/**/

    			if (p1_x < p2_x) {
        			if (equal_to_precision(alpha_min,alpha_x_min,PRECISION)==1)
            				i_min=1;
        			else
            				i_min = (int) ceil(phi(alpha_min, p1_x, p2_x, b_x, d_x));

			        if (equal_to_precision(alpha_max,alpha_x_max,PRECISION)==1)
			            i_max = N_x - 1;
        			else
            				i_max = (int) floor( phi(alpha_max, p1_x, p2_x, b_x, d_x));

				alpha_x=alpha_fn(i_min, p1_x, p2_x, b_x, d_x);
	    		}

    			else {
        			if (equal_to_precision(alpha_min,alpha_x_min,PRECISION)==1)
	            			i_max=N_x-2;
        			else
	            			i_max = (int) floor(phi(alpha_min, p1_x, p2_x, b_x, d_x));

		        	if (equal_to_precision(alpha_max,alpha_x_max,PRECISION)==1)
	            			i_min = 0;
        			else
	            			i_min = (int) ceil( phi(alpha_max, p1_x, p2_x, b_x, d_x));

				alpha_x=alpha_fn(i_max, p1_x, p2_x, b_x, d_x);
			}


    			if (p1_y < p2_y) {
        			if (equal_to_precision(alpha_min,alpha_y_min,PRECISION)==1)
            				j_min=1;
        			else
            				j_min = (int) ceil(phi(alpha_min, p1_y, p2_y, b_y, d_y));


        			if (equal_to_precision(alpha_max, alpha_y_max,PRECISION)==1)
        				j_max = N_y - 1;
        			else
            				j_max = (int) floor( phi(alpha_max, p1_y, p2_y, b_y, d_y));

				alpha_y=alpha_fn(j_min, p1_y, p2_y, b_y, d_y);
			}

    			else {

        			if (equal_to_precision(alpha_min,alpha_y_min,PRECISION)==1)
            				j_max=N_y-2;
        			else
            				j_max = (int) floor(phi(alpha_min, p1_y, p2_y, b_y, d_y));


			        if (equal_to_precision(alpha_max, alpha_y_max, PRECISION)==1)
            				j_min = 0;
        			else
            				j_min = (int) ceil( phi(alpha_max, p1_y, p2_y, b_y, d_y));

				alpha_y=alpha_fn(j_max, p1_y, p2_y, b_y, d_y);
			}



    			if (p1_z < p2_z) {
        			if (equal_to_precision(alpha_min,alpha_z_min,PRECISION)==1)
            				k_min=1;
        			else
            				k_min = (int) ceil(phi(alpha_min, p1_z, p2_z, b_z, d_z));


        			if (equal_to_precision(alpha_max, alpha_z_max,PRECISION)==1)
            				k_max = N_z - 1;
        			else
            				k_max = (int) floor( phi(alpha_max, p1_z, p2_z, b_z, d_z));

				alpha_z=alpha_fn(k_min, p1_z, p2_z, b_z, d_z);
			}

    			else {

        			if (equal_to_precision(alpha_min,alpha_z_min,PRECISION)==1)
            				k_max=N_z-2;
        			else
            				k_max = (int) floor(phi(alpha_min, p1_z, p2_z, b_z, d_z));


        			if (equal_to_precision(alpha_max, alpha_z_max, PRECISION)==1)
        	    			k_min = 0;
        			else
            				k_min = (int) ceil( phi(alpha_max, p1_z, p2_z, b_z, d_z));

				alpha_z=alpha_fn(k_max, p1_z, p2_z, b_z, d_z);
			}


			N_p=(i_max - i_min +1) + (j_max - j_min + 1) + (k_max - k_min + 1);


    			i=(int) floor( phi( (min3_dbl(alpha_x, alpha_y, alpha_z) + alpha_min)/2, p1_x, 
p2_x, b_x, d_x) );
    			j=(int) floor( phi( (min3_dbl(alpha_x, alpha_y, alpha_z) + alpha_min)/2, p1_y, 
p2_y, b_y, d_y) );
    			k=(int) floor( phi( (min3_dbl(alpha_x, alpha_y, alpha_z) + alpha_min)/2, p1_z, 
p2_z, b_z, d_z) );

    			alpha_x_u = d_x/fabs(p2_x-p1_x);
			if (p1_x < p2_x)
        			i_u=1;
    			else
        			i_u=-1;

    			alpha_y_u = d_y/fabs(p2_y-p1_y);
    			if (p1_y < p2_y)
        			j_u=1;
    			else
        			j_u=-1;

			alpha_z_u = d_z/fabs(p2_z-p1_z);
    			if (p1_z < p2_z)
        			k_u=1;
    			else
        			k_u=-1;


    			alpha_c=alpha_min;

    			/* should possibly use N_v here*/
    			for (n_count=1; n_count<N_p+1;n_count++) {


    				/*%PROBLEM. sometimes i or j increase too much. fix!
    				%this seems to work. stills seems odd tho */
				if (0>i || i>im_size-1 || 0>j || j>im_size-1 || 0>k || k>im_size-1) {
     				/*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, 
jacobs_ray);*/
		if(testing == 1)
		  printf("returns here (2)");fflush(NULL);
    					return;
    				}


        			/* x smallest*/
				if (alpha_x <= alpha_y && alpha_x <= alpha_z) {
             				/* ray intersects pixel(i,j) with length l_ij */

				    	ray_index[(*n_entries)] = k*im_size*im_size + j*im_size + i;
					ray_data[(*n_entries)++] = (alpha_x-alpha_c);

            				i=i+i_u;
            				alpha_c=alpha_x;
            				alpha_x = alpha_x + alpha_x_u;
	    			}

				/* y smallest*/
				else if (alpha_y <= alpha_y && alpha_y <= alpha_z) {
            				/* ray intersects pixel(i,j) with length l_ij */

					ray_index[(*n_entries)] = k*im_size*im_size + j*im_size + i;
					ray_data[(*n_entries)++] = (alpha_y-alpha_c);

            				j=j+j_u;
            				alpha_c=alpha_y;
            				alpha_y = alpha_y + alpha_y_u;
	    			}

				/* z smallest*/
				else {
            				/* ray intersects pixel(i,j) with length l_ij */

				    	ray_index[(*n_entries)] = k*im_size*im_size + j*im_size + i;
					ray_data[(*n_entries)++] = (alpha_z-alpha_c);

            				k=k+k_u;
			        	alpha_c=alpha_z;
            				alpha_z = alpha_z + alpha_z_u;
	    			}


 			} /* end of for loop though N_p */

		} /* of alpha_min < alpha_max */

 /*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/
	if(testing == 1)
	  printf(":jacobs_ray_3d done:");fflush(NULL);

	return;

	} /* of points px!=px && py!=py */

}



/***********************************************************************/



void jacobs_ray_2d(int im_size, double p1_x, double p1_y, double p2_x, double
		p2_y, int *ray_index_i, int *ray_index_j, double *ray_data, int *n_entries, double voxel_size)
{

	int N_x, N_y, N_p;
	double b_x, b_y, d_x, d_y, d_conv; /*, d_conv_alpha; */

	int i,j;

	double alpha_x_min, alpha_y_min, alpha_x_max, alpha_y_max, alpha_min, 
alpha_max, alpha_x, alpha_y, alpha_c;
	double alpha_x_u, alpha_y_u;
	double l_ij;
	int i_min, j_min, i_max, j_max, n_count, i_u, j_u;

	*n_entries=0;

		
	b_x=-im_size*voxel_size/2;
	b_y=-im_size*voxel_size/2;

	d_x=voxel_size;
	d_y=voxel_size;

	/* use total lengh=alpha_max-alpha_min instead, to get everage, not sum. */
	d_conv=sqrt( (p1_x-p2_x)*(p1_x-p2_x) + (p1_y-p2_y)*(p1_y-p2_y) );/**/

	N_x=im_size+1;
	N_y=im_size+1;

if (equal_to_precision(p1_x,p2_x,PRECISION)==1) {
    i=(int)floor( phi(0, p1_x, p2_x, b_x, d_x));
    /* or ceil? */

    /*may as well assume ray starts and ends outside grid as this will be
    the case whenever this is used */
	if (0 <= i && i <= im_size-1)
    		for (j=0; j< im_size; j++) {
			/*ray_index[(*n_entries)] = j* im_size + i;*/
			ray_index_i[(*n_entries)] = i;
			ray_index_j[(*n_entries)] = j;
			ray_data[(*n_entries)++] = 1.0 * voxel_size / d_conv;
		}
	else {
		/* leave empty */
	}

	/*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/
	return;
}

else if (equal_to_precision(p1_y,p2_y,PRECISION)==1) {
	j=(int)floor( phi(0, p1_y, p2_y, b_y, d_y));
	/* or ceil? */

	/*may as well assume ray starts and ends outside grid as this will be
    	%the case whenever this is used */
	if (0 <= j && j <= im_size-1)
    		for (i=0; i< im_size; i++) {
			/*ray_index[(*n_entries)] = j* im_size + i;*/
			ray_index_i[(*n_entries)] = i;
			ray_index_j[(*n_entries)] = j;
			ray_data[(*n_entries)++] = 1.0 * voxel_size / d_conv;
		}
	else {
		/* leave as zero */
	}

	/*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/
	return;
}

else {
    alpha_x_min=min_dbl(alpha_fn(0, p1_x, p2_x, b_x, d_x), alpha_fn(N_x-1, p1_x, 
p2_x, b_x, d_x));
    alpha_x_max=max_dbl(alpha_fn(0, p1_x, p2_x, b_x, d_x), alpha_fn(N_x-1, p1_x, 
p2_x, b_x, d_x));


    alpha_y_min=min_dbl(alpha_fn(0, p1_y, p2_y, b_y, d_y), alpha_fn(N_y-1, p1_y, 
p2_y, b_y, d_y));
    alpha_y_max=max_dbl(alpha_fn(0, p1_y, p2_y, b_y, d_y), alpha_fn(N_y-1, p1_y, 
p2_y, b_y, d_y));

    alpha_min=max_dbl(alpha_x_min, alpha_y_min);
    alpha_max=min_dbl(alpha_x_max, alpha_y_max);

if (alpha_min < alpha_max) {

    /* use total lengh=alpha_max-alpha_min instead, to get everage, not sum. */
    /* d_conv_alpha=alpha_max-alpha_min; /**/




    if (p1_x < p2_x) {
        if (equal_to_precision(alpha_min,alpha_x_min,PRECISION)==1)
            i_min=1;
        else
            i_min = (int)ceil(phi(alpha_min, p1_x, p2_x, b_x, d_x));


        if (equal_to_precision(alpha_max,alpha_x_max,PRECISION)==1)
            i_max = N_x - 1;
        else
            i_max = (int)floor( phi(alpha_max, p1_x, p2_x, b_x, d_x));

	alpha_x=alpha_fn(i_min, p1_x, p2_x, b_x, d_x);
	    }

    else {
        if (equal_to_precision(alpha_min,alpha_x_min,PRECISION)==1)
            i_max=N_x-2;
        else
            i_max = (int)floor(phi(alpha_min, p1_x, p2_x, b_x, d_x));


        if (equal_to_precision(alpha_max,alpha_x_max,PRECISION)==1)
            i_min = 0;
        else
            i_min = (int)ceil( phi(alpha_max, p1_x, p2_x, b_x, d_x));

	alpha_x=alpha_fn(i_max, p1_x, p2_x, b_x, d_x);
	}


    if (p1_y < p2_y) {
        if (equal_to_precision(alpha_min,alpha_y_min,PRECISION)==1)
            j_min=1;
        else
            j_min = (int)ceil(phi(alpha_min, p1_y, p2_y, b_y, d_y));


        if (equal_to_precision(alpha_max, alpha_y_max,PRECISION)==1)
            j_max = N_y - 1;
        else
            j_max = (int)floor( phi(alpha_max, p1_y, p2_y, b_y, d_y));

	alpha_y=alpha_fn(j_min, p1_y, p2_y, b_y, d_y);
	}

    else {

        if (equal_to_precision(alpha_min,alpha_y_min,PRECISION)==1)
            j_max=N_y-2;
        else
            j_max = (int)floor(phi(alpha_min, p1_y, p2_y, b_y, d_y));


        if (equal_to_precision(alpha_max, alpha_y_max, PRECISION)==1)
            j_min = 0;
        else
            j_min = (int)ceil( phi(alpha_max, p1_y, p2_y, b_y, d_y));

	alpha_y=alpha_fn(j_max, p1_y, p2_y, b_y, d_y);
	}


    N_p=(i_max - i_min +1) + (j_max - j_min + 1);


    i=(int)floor( phi( (min_dbl(alpha_x, alpha_y) + alpha_min)/2, p1_x, p2_x, b_x, d_x) 
);
    j=(int)floor( phi( (min_dbl(alpha_x, alpha_y) + alpha_min)/2, p1_y, p2_y, b_y, d_y) 
);

    alpha_x_u = d_x/fabs(p2_x-p1_x);
    if (p1_x < p2_x)
        i_u=1;
    else
        i_u=-1;

    alpha_y_u = d_y/fabs(p2_y-p1_y);
    if (p1_y < p2_y)
        j_u=1;
    else
        j_u=-1;


    alpha_c=alpha_min;

    /* should possibly use N_v here*/
    for (n_count=1; n_count<N_p+1;n_count++)
    {


    /*%PROBLEM. sometimes i or j increase too much. fix!
    %this seems to work. stills seems odd tho */
if (0>i || i>im_size-1 || 0>j || j>im_size-1) {
     /*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/
    return;
    }


        if (alpha_x < alpha_y) {
             /* ray intersects pixel(i,j) with length l_ij */
             l_ij=(alpha_x-alpha_c); 

            /* remove tiny rounding errors by rounding to 8 d.p.
            l_ij=round(l_ij*10^8)/10^8;*/


	    	/*ray_index[(*n_entries)] = j* im_size + i;*/
		ray_index_i[(*n_entries)] = i;
		ray_index_j[(*n_entries)] = j;

		ray_data[(*n_entries)++] = l_ij; /* * d_conv_alpha / d_conv; /**/

            i=i+i_u;
            alpha_c=alpha_x;
            alpha_x = alpha_x + alpha_x_u;
	    }
        else {
            /* ray intersects pixel(i,j) with length l_ij */
            l_ij=(alpha_y-alpha_c); 

            /* remove tiny rounding errors by rounding to 8 d.p.
            l_ij=round(l_ij*10^8)/10^8; */


	    	/*ray_index[(*n_entries)] = j* im_size + i;*/
		ray_index_i[(*n_entries)] = i;
		ray_index_j[(*n_entries)] = j;
		ray_data[(*n_entries)++] = l_ij; /* * d_conv_alpha / d_conv; /**/

            j=j+j_u;
            alpha_c=alpha_y;
            alpha_y = alpha_y + alpha_y_u;
	    }

	    } /* of loop through N_p*/
 } /* of alpha_min < alpha_max */

 /*sort_and_set(tmp_arr_index, tmp_arr_data, &tmp_counter, jacobs_ray);*/
return;

} /* of points px!=px && py!=py */

}




double alpha_fn(int n, double p1, double p2, double b, double d)
{
    return ( (b+n*d) - p1)/(p2-p1);
}


double phi(double alpha, double p1, double p2, double b, double d)
{
    return ( p(alpha, p1, p2)-b)/d;
}


double p(double alpha, double p1, double p2)
{
    return p1+alpha*(p2-p1);
}



int equal_to_precision(double x, double y, double prec)
{
   return fabs(x-y) < prec;
}



#if 0 == 1
void qsort_indexed_list(int *index, double *values, int l, int r)
/* to sort the entries in index ascending, and entries in values to match
[1 3 2			[1 2 3
   a b c]    becomes	 a c b] */
{
	/*mexPrintf("qsort, left: %d, right: %d \n", l, r);*/

	int l1, r1;

	if (l < r) {
		l1=l;
		r1=r;

		do {
		while (l1 < r && index[l1-1] <= index[l-1])
			l1++;
		while(l < r1 && index[r1-1] >= index[l-1])
			r1--;
		if (l1 < r1) {
			swop_int( &index[l1-1], &index[r1-1]);
			swop_dbl( &values[l1-1], &values[r1-1]);
			}
		} while (l1 < r1);

		swop_int( &index[l-1], &index[r1-1]);
		swop_dbl( &values[l-1], &values[r1-1]);

		qsort_indexed_list(index, values, l, r1-1);
		qsort_indexed_list(index, values, r1+1, r);
	}
}

void swop_int(int *a, int *b)
/* swops the values of a and b */
{
	double t;
	t=*a;
	*a=*b;
	*b=t;
}

void swop_dbl(double *a, double *b)
/* swops the values of a and b */
{
	double t;
	t=*a;
	*a=*b;
	*b=t;
}

#endif

