%
% FUNCTION [samXYZ,lab2sam] = gtGeoLab2Sam(labXYZ,omega,labgeo,samgeo,freevec_flag)
%
% LAB -> SAMPLE coordinate transformation
%
% Given the LAB coordinates of a set of points, transforms them 
% into SAMPLE coordinates according to an omega rotation. Omega is usually
% positive.
% It optionally creates the transformation matrix.
%
% INPUT  
%   labXYZ         - coordinates in the Lab reference in pixels (n,3)
%   omega          - omega angles in degrees (n,1)
%   labgeo.rotpos  - rotation axis position in Lab coordinates (1,3)
%   labgeo.rotdir  - rotation axis direction (right-handed rotation)
%   samgeo.orig    - origin of the sample reference in Lab coordinates (1,3)
%   samgeo.voxsize - [x,y,z] voxel sizes in lab units
%   samgeo.lab2sam - optional; transformation matrix from Lab to Sample
%                    coordinates (3,3)
%   freevec_flag   - 0 for fixed vectors (e.g. location);
%                    1 for free vectors (e.g. orientation)
%
% OUTPUT 
%   samXYZ         - coordinates in the Sample reference in no. of voxels (n,3)
%   lab2sam        - coordinate transformation matrix
%

function [samXYZ,lab2sam] = gtGeoLab2Sam(labXYZ,omega,labgeo,samgeo,freevec_flag)

n = size(labXYZ,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Change of reference from Lab to Sample
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Lab to Sample coordinates:
%   Note that Lab is an orthonormal basis by definition, but Sample may not be !!
%   Therefore needs calculation of the inverse, and not only transpose.
if ~isfield(samgeo,'lab2sam') || (isfield(samgeo,'lab2sam') && isempty(samgeo.lab2sam))
    sam2lab = [samgeo.dirx*samgeo.voxsize(1);...
               samgeo.diry*samgeo.voxsize(2);...
               samgeo.dirz*samgeo.voxsize(3)];
           
    lab2sam = inv(sam2lab);
else
    lab2sam = samgeo.lab2sam;
end

%%%%%%%%%%%%%%%%%%%%%
%% Rotation in Lab
%%%%%%%%%%%%%%%%%%%%%

% Tensor which rotates with omega from Lab back to Sample:
rotcomp    = gtMathsRotationMatrixComp(labgeo.rotdir, 'row');
rottensors = gtMathsRotationTensor(-omega, rotcomp);


if freevec_flag    % free vectors
    
    % Multiply element-wise
    pro = reshape(repmat(labXYZ,1,3)',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose to get rotated vector
    labXYZ_rot = reshape(sum(pro,1),3,n)';

    % Change ref. base
    samXYZ = labXYZ_rot*lab2sam;

else % offset vectors
    
    % Vectors relative to rotation axis position
    labXYZrel = labXYZ - repmat(labgeo.rotpos,n,1);
    
    % Apply rottensor element-wise
    pro = reshape(repmat(labXYZrel,1,3)',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose, set offset back to get rotated vector
    labXYZ_rot = reshape(sum(pro,1),3,n)' + repmat(labgeo.rotpos,n,1);
 
    % Change ref. base
    samXYZ = (labXYZ_rot - repmat(samgeo.orig,n,1))*lab2sam;
    
end


