%
% FUNCTION [labXYZ,sam2lab] = gtGeoSam2Lab(samXYZ,om,labgeo,samgeo,freevec_flag)
%
% SAMPLE -> LAB coordinate transformation
%
% Given the SAMPLE coordinates of a set of points, transforms them 
% into LAB coordinates according to an omega rotation. Omega is usually
% positive.
% It optionally creates the transformation matrix.
%
% INPUT  
%   samXYZ         - coordinates in the Sample reference in no. of voxels (n,3)
%   om             - omega angles in degrees (n,1)
%   labgeo.rotpos  - rotation axis position in Lab coordinates (1,3)
%   labgeo.rotdir  - rotation axis direction (right-handed rotation)
%   samgeo.orig    - origin of the sample reference in Lab coordinates (1,3)
%   samgeo.voxsize - [x,y,z] voxel sizes in lab units
%   samgeo.sam2lab - optional; transformation matrix from Sample to Lab
%                    coordinates (3,3)
%   freevec_flag   - 0 for fixed vectors (e.g. location);
%                    1 for free vectors (e.g. orientation)
%
% OUTPUT 
%   labXYZ         - row vector of SAMPLE coordinates (n,3)
%   sam2lab        - coordinate transformation matrix (field added if it 
%                    does not yet exists)
%

function [labXYZ,sam2lab] = gtGeoSam2Lab(samXYZ,om,labgeo,samgeo,freevec_flag)

n = size(samXYZ,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Change of reference from Sample to Lab
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Sample to Lab coordinates:
%   (Lab is an orthonormal basis by definition.)
if ~isfield(samgeo,'sam2lab')
    sam2lab = [samgeo.dirx*samgeo.voxsize(1);...
               samgeo.diry*samgeo.voxsize(2);...
               samgeo.dirz*samgeo.voxsize(3)];
else
    sam2lab = samgeo.sam2lab;
end

%%%%%%%%%%%%%%%%%%%%%
%% Rotation in Lab
%%%%%%%%%%%%%%%%%%%%%

% Tensor which rotates with omega from Sample to Lab
% (omega is usually positive):
rotcomp    = gtMathsRotationMatrixComp(labgeo.rotdir, 'row');
rottensors = gtMathsRotationTensor(om, rotcomp);

if freevec_flag    % free vectors

    % Vectors in Lab reference at the origin before rotation
    labXYZprerot = samXYZ*sam2lab;

    % Multiply element-wise to avoid loop
    pro = reshape(repmat(labXYZprerot,1,3)',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose
    labXYZ = reshape(sum(pro,1),3,n)';
    
else % offset vectors

    % Vectors relative to rot. axis position in Lab reference
    labXYZprerot = samXYZ*sam2lab + repmat(samgeo.orig-labgeo.rotpos,n,1);
   
    % Apply rottensor element-wise to avoid loop
    pro = reshape(repmat(labXYZprerot,1,3)',3,n*3).*reshape(rottensors,3,n*3);

    % Sum, reshape, transpose, set offset back
    labXYZ = reshape(sum(pro,1),3,n)' + repmat(labgeo.rotpos,n,1);
     
end



