%
% FUNCTION [xyzB,samA2samB] = gtGeoSam2Sam(xyzA,samgeoA,samgeoB,freevec_flag)
%
% SAMPLE -> SAMPLE coordinate transformation
%
% Given the coordinates of a set of points in reference SampleA, transforms
% them into coordinates in reference SampleB.
% It optionally creates the transformation matrix.
%
% INPUT  
%   xyzA           - coordinates in reference SampleA in no. of voxels (n,3)
%   samgeoA.orig   - origin of reference A in Lab coordinates (1,3)
%   samgeoB.orig   - origin of reference B in Lab coordinates (1,3)
%   freevec_flag   - 0 for fixed vectors (e.g. location);
%                    1 for free vectors (e.g. orientation)
%   samgeoA.voxsize - [x,y,z] voxel sizes A in lab units
%                     (required if samgeoA.sam2lab is not defined; )
%   samgeoB.voxsize - [x,y,z] voxel sizes B in lab units
%                     (required if samgeoB.lab2sam is not defined; )
%
% OPTIONAL INPUT
%   samgeoA.sam2lab - matrix for transformation from SampleA to Lab coordinates (3,3)
%   samgeoB.lab2sam - matrix for transformation from Lab to SampleB coordinates (3,3)
%
% OUTPUT 
%   xyzB      - coordinates in reference SampleB in no. of voxels (n,3)
%   samA2samB - coordinate transformation matrix from Sample A to B (3,3)
%

function [xyzB,samA2samB] = gtGeoSam2Sam(xyzA,samgeoA,samgeoB,freevec_flag)

n = size(xyzA,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Change of reference from SampleA to Lab
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Sample to Lab coordinates:
%   (Lab is an orthonormal basis by definition.)
if ~isfield(samgeoA,'sam2lab')
    samgeoA.sam2lab = [samgeoA.dirx*samgeoA.voxsize(1);...
                       samgeoA.diry*samgeoA.voxsize(2);...
                       samgeoA.dirz*samgeoA.voxsize(3)];
end


% Vectors in Lab reference

if freevec_flag    % free vectors

    labXYZ = xyzA*samgeoA.sam2lab;
   
else % offset vectors
    
    labXYZ = xyzA*samgeoA.sam2lab + repmat(samgeoA.orig,n,1);

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Change of reference from Lab to SampleB
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Lab to Sample coordinates:
%   Note that Lab is an orthonormal basis by definition, but Sample may not be !!
%   Therefore needs calculation of the inverse, and not only transpose.
if ~isfield(samgeoB,'lab2sam')
    sam2lab = [samgeoB.dirx*samgeoB.voxsize(1);...
               samgeoB.diry*samgeoB.voxsize(2);...
               samgeoB.dirz*samgeoB.voxsize(3)];
    % Inverse       
    samgeoB.lab2sam = inv(sam2lab);
end


% Vectors in SampleB reference

if freevec_flag   % free vectors
   
    xyzB = labXYZ*samgeoB.lab2sam;

else % offset vectors
    
    xyzB = (labXYZ - repmat(samgeoB.orig,n,1))*samgeoB.lab2sam;
    
end

samA2samB = samgeoA.sam2lab*samgeoB.lab2sam;

