function gtINDEXDrawGrainCubes(grains, parameters, numbered, hlight, strainscale)
% function gtINDEXDrawGrainCubes(grains, parameters, numbered, hlight, strainscale)
%
% Draws a figure representing grain size, location and orientation by cubes 
% (unit cells in a cubic lattice) based on the their Rodrigues vectors.
%
% USAGE  gtDrawGrainCubes
%        gtDrawGrainCubes(grains(1:10),[],[],1)  
%
% OPTIONAL INPUT  
%   grains:      grain structure from indexing
%   parameters:  as in parameters file (required field: labgeo)
%   numbered:    if true, the grain id-s for each grain are shown
%   hlight:      grain id-s to be highlighted
%   strainscale: a multiplicative factor for the strain components
%                     for better visibility (if strain data available)


if ~exist('grains','var') || isempty(grains)
   disp('Loading grain data from 4_grains/phase_01/index.mat ...')
   tmp = load('4_grains/phase_01/index.mat','grain');
   grains = tmp.grain;
end

if ~exist('parameters','var') || isempty(parameters)
    disp('Loading parameters from parameters.mat ...')
    load('parameters.mat');
end

if ~exist('hlight','var') || isempty(hlight)
    hlight = [];
end

if ~exist('numbered','var') || isempty(numbered)
    numbered = [];
end

if ~exist('strainscale','var') || isempty(strainscale)
    strainscale = 50;
end

sample.rad = parameters.labgeo.samenvrad;
sample.bot = parameters.labgeo.samenvbot;
sample.top = parameters.labgeo.samenvtop;

nof_grains = length(grains);

fs = 20;
samradmargin = 0.05;
samtopmargin = 0.2;
pixelsize = (parameters.labgeo.pixelsizeu + parameters.labgeo.pixelsizev)/2;


figure

colormap(jet(1000))
caxis([-0.01 0.01])
cb = colorbar('FontSize', fs, 'ytick', -0.01:0.005:0.01);
%cb=colorbar('ytick',-0.01:0.002:0.01);
%cbpos=get(cb,'Position');
cbpos = [0.01 0.2 0.025 0.6];
set(cb,'Position',cbpos);

set(gca,'xtick',[],'xcolor','w')
set(gca,'ytick',[],'ycolor','w')
set(gca,'ztick',[],'zcolor','w')

%set(gca,'Position',[0.1 0 0.9 1])
set(gca,'Position',[0 0 1 1])

view(3);

axhor = 0.01*floor((sample.rad*(1+samradmargin))/0.01);
axver = 0.01*floor((max(abs(sample.top),abs(sample.bot))*(1+samtopmargin))/0.01);
%axis([-axhor axhor -axhor axhor -axver axver])

%xlabel('X')
%ylabel('Y')
%zlabel('Z')
hold on

get(gca,'CameraPosition');
get(gca,'CameraViewAngle');

t = 1:360;
circx = cosd(t)*sample.rad;
circy = sind(t)*sample.rad;
circbotz(1:360) = sample.bot;
circtopz(1:360) = sample.top;

plot3(circx, circy, circbotz, 'k-', 'Linewidth', 1)
plot3(circx, circy, circtopz, 'k-', 'Linewidth', 1)

plot3([0 0],[0 0],[sample.bot sample.top],'k.-.','Linewidth',1)

plot3([-sample.rad -sample.rad],[0 0],[sample.bot sample.top],'k.-','Linewidth',1)
plot3([0 0],[-sample.rad -sample.rad],[sample.bot sample.top],'k.-','Linewidth',1)
plot3([sample.rad sample.rad],[0 0],[sample.bot sample.top],'k.-','Linewidth',1)
plot3([0 0],[sample.rad sample.rad],[sample.bot sample.top],'k.-','Linewidth',1)


% beam direction: 
%plot3([-170 170],[0 0],[0 0],'r-','Linewidth',2)


cube_vertices = [0 0 0; 1 0 0; 1 1 0; 0 1 0; 0 0 1; 1 0 1; 1 1 1; 0 1 1] - ...
                0.5*repmat([1 1 1],8,1);

% Cubed face directions
%             +yz      -yz      -xz       +xz       +xy      -xy       
%             +x       -x       -y        +y        +z       -z
cube_faces = [2 3 7 6; 4 1 5 8; 1 2 6 5;  3 4 8 7;  5 6 7 8; 1 2 3 4];

cube_planes = [1 0 0; -1 0 0; 0 -1 0; 0 1 0; 0 0 1; 0 0 -1];

for ii = 1:nof_grains
    
    if ismember(ii,hlight)
        hl = true;
    else
        hl = false;
    end
    
    sfPlotGrainCube(grains{ii}, cube_vertices, cube_faces, ...
                    hl, strainscale, pixelsize)
    
    if numbered
        text(grains{ii}.center(1), -axhor,grains{ii}.center(3),num2str(ii), 'Color','c')
        text(axhor, grains{ii}.center(2), grains{ii}.center(3), num2str(ii), 'Color','c')
    end
end


axis equal
axis([-axhor axhor -axhor axhor -axver axver])


end % of main function



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SUB-FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sfPlotGrainCube(grain, cube_vertices, cube_faces, ...
                         hl, strainscale, pixelsize)

ga = 0.5*pixelsize*(grain.stat.bbxsmean + grain.stat.bbysmean)/2;

if isfield(grain,'centre')
	gc = grain.centre;
else
	gc = grain.center;
end


% if hl
%   facecolors=cool(8);
%   facecolors=facecolors(end-5:end,:);
% else
%   facecolors=copper(8);
%   facecolors=facecolors(end-5:end,:);
% end  


% ELONGATION ALONG LOADING AXIS
% 
ggrain = Rod2g(grain.R_vector);
grain_vertices = ga*cube_vertices*ggrain';

if isfield(grain,'strain') && ~any(isnan(grain.strain.strainT(:)))
	grain_vertices_st = repmat(gc,8,1) + ...
                        ((eye(3) + grain.strain.strainT*strainscale)*grain_vertices')';
	facecolor = grain.strain.strainT(3,3);
else
	grain_vertices_st = repmat(gc,8,1) + (eye(3)*grain_vertices')';
	facecolor = 0;
end


facecolor = repmat(facecolor,6,1);
%facecolor=facecolor.*repmat([1.0; 0.98; 0.96; 0.94; 0.92; 0.90],1,3);



% SHEAR STRAIN ON THE CUBE CELL FACES

% T=Rod2g(grain.R_vector); % T rotation tensor in lab system (align the axes with the crystal axes); 
% 
% ECRYST=T'*grain.strain.ST*T;  % strain tensor in the crystal reference
% 
% facecolor=zeros(6,1);
% facecolor([1,2])=sqrt(ECRYST(1,2)^2+ECRYST(1,3)^2); % xy & xz
% facecolor([3,4])=sqrt(ECRYST(2,1)^2+ECRYST(2,3)^2); % yx & yz
% facecolor([5,6])=sqrt(ECRYST(3,1)^2+ECRYST(3,2)^2); % zx & zy
% 
% ST=grain.strain.ST;
% 
% grain_vertices=ga*cube_vertices*T';
% grain_vertices_st=repmat(gc,8,1)+((eye(3)+ST*strainscale)*grain_vertices')';
% 


if hl
    patch('Vertices', grain_vertices_st, 'Faces', cube_faces , ...
        'FaceVertexCData', facecolor, 'FaceColor', 'flat','EdgeColor','r','LineWidth',2);
else
    patch('Vertices', grain_vertices_st, 'Faces', cube_faces , ...
      'FaceVertexCData', facecolor, 'FaceColor', 'flat');
end

% x0=[1 0 0];
% y0=[0 1 0];
% z0=[0 0 1];
% 
% xcr=T*x0';
% ycr=T*y0';
% zcr=T*z0';
% 
% 
%  quiver3(gc(1),gc(2),gc(3),xcr(1),xcr(2),xcr(3),200,'r')
%  quiver3(gc(1),gc(2),gc(3),ycr(1),ycr(2),ycr(3),200,'b')
%  quiver3(gc(1),gc(2),gc(3),zcr(1),zcr(2),zcr(3),200,'k')
%  
%  grain_vertices=ga*cube_vertices*T';
%  grain_vertices_st=repmat(gc,8,1)+((eye(3))*grain_vertices')';
%  		
%  patch('Vertices',grain_vertices_st,'Faces',cube_faces,...
%       'FaceVertexCData',facecolor,'FaceColor','flat');
%  	
% 		
%    edgevec=[0 1 0];
%    facepos=[1 0 0];
%    edgevec_g=edgevec*T';
%    facepos_g=gc+ga/2*facepos*T';
%    quiver3(facepos_g(1),facepos_g(2),facepos(3),edgevec_g(1),edgevec_g(2),edgevec_g(3),100,'c');
 
  
      
end  % of sfPlotGrainCube


