function [grain, unindexed] = gtIndexBuildGrains(remaining_in, tot, ...
                            tol_build, cryst, samenv)
%
% FUNCTION [grain, unindexed] = gtIndexBuildGrains(remaining_in, tot, ...
%                               tol_build, cryst, samenv)
% 
% 1st/3 Step of the iterative indexing strategy. Finds grains among the 
% reflections, returns the grains with their orientation, position and
% statistical data on the reflections.
% The signed hkl indices of the reflections are determined.
% Unindexed reflection id-s are returned in 'unindexed'.  
%
% INPUT
%
%  remaining_in - yet unindexed reflection id-s (as in tot.id)
%  tot.         - all reflection data (including indexed and unindexed)
%  tol_build.   - tolerances for building grains;
%                 (as in paramers.index.strategy.b.beg or b.end)
%  cryst.       - crystallography and other data
%  samenv.      - sample envelope in the Sample reference
%
% OUTPUT
%  grain        - grain data (cell vector)
%  unindexed    - remaining unindexed reflection id-s
%


                        
fprintf('\n BUILDING GRAINS...\n\n')
disp(tol_build)
disp(' ')
tic

remaining  = remaining_in;
nof_grains = 0;
unindexed  = [];
grain      = [];


% Loop until there are reflections left to check

while length(remaining) >= 1
    
    % Current reflection to be tested
    act = gtIndexSelectRefs(remaining(1), tot);     
    
    % Remaining candidate reflections
    cand_rem = gtIndexSelectRefs(remaining(2:end), tot); 
    
    % Tolerance for distance between diffraction paths is set equal to the 
    % approximate grain size times a constant, but is limited to a max.
    tol_build.dist = min([act.bbxs, act.bbys]) * cryst.pixelsize * ...
                     tol_build.distf ;
    tol_build.dist = min(tol_build.dist, tol_build.distmax) ;
    
    % Check pair consistency of reflections (all candidates against
    % actual).
    % 'paircons' is a vector of the linear indices of consistent 
    % reflections in cand_rem.
    [paircons, isecs] = gtIndexCheckPairCons(act, cand_rem, ...
                        tol_build, samenv, cryst.ACM, tol_build.ming-1);
    
    % If there are enough pair consistent candidates
    if length(paircons) >= tol_build.ming-1
        
        % Load reflection data for candidates
        cand_paircons = gtIndexSelectRefs([act.id; cand_rem.id(paircons)], tot);
        
        % Find a consistent group among those candidates (check group 
        % consistency)
        [pgroup, grainfound] = gtIndexFindGroupInCand(cand_paircons, ...
                               isecs, tol_build, samenv, cryst);
        
        % If there is a consistent group found
        if ~isempty(grainfound)
           
            nof_grains           = nof_grains + 1;
            grain{nof_grains}    = grainfound;
            grain{nof_grains}.id = nof_grains;
            
            fprintf('Found new grain #%d:   %s\n', nof_grains, ...
                    num2str(grain{nof_grains}.pairid))
            
            todel = cand_paircons.id(pgroup);
            
        % No consistent group was found for the actual reflection. 
        % Put it in unindexed set.
        else
            unindexed = [unindexed; act.id];
            todel = act.id;
        end
        
    % If there are not enough consistent candidates, put actual reflection 
    % in unindexed set
    else
        unindexed = [unindexed; act.id];
        todel = act.id;
    end
    
    % Delete reflections which have been dealt with from remaining
    remaining(ismember(remaining,todel)) = [];
    
end


disp(' ')
toc


end % of function
