function [paircons, pofinters] = gtIndexCheckPairCons(act, cand, tol, ...
                                                      samenv, ACM, minn)
%
% FUNCTION  [paircons, pofinters] = gtIndexCheckPairCons(act, cand, ...
%                                                        tol, samenv, ACM)
%
% Checks pair consistency of reflection 'act' against a set of others 
% in 'cand'.
%
% INPUT
%   act    - the actual reflection to be tested against
%   cand   - a set of candidate reflections to be checked
%   tol    - tolerances
%   samenv - sample envelope
%   ACM    - angular consistency matrix
%   minn   - minimum number of consistent reflections to find
%
% OUTPUT
%   paircons  - a vector of logicals; true where 'cand' is pair consistent
%   pofinters - point of 'intersection' between diff. paths of 'act' and
%               consistent 'cand'-s pair wise; empty if found less than 
%               'minn' consistent reflections
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Check bbox sizes and intensity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% initial indices of consistent reflections 
paircons = (1:length(cand.id))';  


% bbox y size close enough?
cons = (act.bbys/tol.bbys < cand.bbys) & (cand.bbys < act.bbys*tol.bbys);
paircons(~cons) = []; 

if length(paircons) < minn
	pofinters = [];
    return
end


% bbox x size close enough?
cons = (act.bbxs/tol.bbxs < cand.bbxs(paircons)) & (cand.bbxs(paircons) < act.bbxs*tol.bbxs);
paircons(~cons) = []; 

if length(paircons) < minn
	pofinters = [];
    return
end


% integrated intensities close enough?
cons = (act.int/tol.int < cand.int(paircons)) & (cand.int(paircons) < act.int*tol.int);
paircons(~cons) = [];

if length(paircons) < minn
	pofinters = [];
    return
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Check distance and location
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% spatial distance small enough?
cons = gtMathsLinesDists([act.ca act.dir],[cand.ca(paircons,:) cand.dir(paircons,:)]) < tol.dist ;
paircons(~cons) = []; 

if length(paircons) < minn
	pofinters = [];
	return
end

% is intersection in the sample volume?
%
% NOTE: the intersection of 2 diff. paths might actually be far from 
% the grain center if they are near being parallel. This could be taken 
% into account here, however, there should be no parallel diff. paths 
% in a grain set.

pofinters = gtMathsLinePairsIntersections([act.ca act.dir], ...
            [cand.ca(paircons,:) cand.dir(paircons,:)]);

cons = gtIndexIsPointInSample(pofinters,samenv);

pofinters(~cons,:) = [];
paircons(~cons) = []; 


if length(paircons) < minn
	pofinters = [];
	return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Check angle
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cons  = false(length(paircons),1);

candhklind  = cand.hklind(paircons);
candpl      = cand.pl(paircons,:);
actpl       = act.pl;

% Angular consistency matrix; hklind{} may have more than one element 
ACM_reduced = ACM(act.hklind{1},:);

% !!! This could be a parfor loop!
for ii = 1:length(paircons)
  ACV = ACM_reduced(:,candhklind{ii});
  ACV = vertcat(ACV{:});
  
  ang = gtMathsVectorsAngles(actpl, candpl(ii,:));
  cons(ii)  = min(abs(ACV-ang)) < tol.ang;
end


pofinters(~cons,:) = [];
paircons(~cons)    = []; 


end % of function

