function [grain, remaining] = gtIndexRefineGrains(grain, tryrefids, tot,...
                                                  tol_b, cryst)
%
% FUNCTION [grain, remaining] = gtIndexRefineGrains(grain, tryrefids, ...
%                                                   tot, tol_b, cryst)
% 
% 3rd/3 Step of the iterative indexing strategy. It loops through all the 
% grains and checks if additional unindexed reflections can be added to 
% them based on tolerances specified for building grains.  
% Grain parameters and statistics for extended grains are updated, 
% reflections are checked for multiplicity and least fitting ones are 
% rejected if multiplicity exceeds the maximum.
%
% INPUT
%
%  grain.       - all grain data (cell vector)
%  tryrefids    - unindexed reflection id-s in 'tot' to be tested 
%  tot.         - all reflection data (including indexed and unindexed)
%  tol_merge.   - tolerances for building grains;
%                 (as in paramers.index.strategy.b.beg or b.end)
%  cryst.       - crystallography and other data
%
% OUTPUT
%  grain        - updated grain data
%  remaining    - remaining unindexed reflection id-s
%

fprintf('\n REFINING GRAINS...\n\n')
disp(tol_b)
disp(' ')
tic

% Grains to delete
gtodel = [];


% all possible normalized signed hkl vectors in cartesian CRYSTAL coordinates
indshkldirs_cry = gtCrystHKL2Cartesian(vertcat(cryst.shklfam{:})', ...
                                       cryst.Bmat)';


% Loop through all grains:

for ii = 1:length(grain);

    % Set distance tolerance using grain statistics
    tol_b.dist = min(3*grain{ii}.stat.dcomstd, tol_b.distmax/2);
    
    % transformation tensor from CRYSTAL to SAMPLE coordinates
    cry2sam = Rod2g(grain{ii}.R_vector);
    
    % normalized signed hkl vectors in cartesian SAMPLE coordinates
    % (transpose needed when using row vectors)
    grain{ii}.shkldirs = indshkldirs_cry*cry2sam'; 
    
    % load data for all candidate reflections
    tryrefs = gtIndexSelectRefs(tryrefids, tot);
    
    % check consistency of bbox sizes, intensity, path distance 
    % and orientation
    goodfits = gtIndexCheckAddRefs(tryrefs, grain{ii}, tol_b, 'tols');
    
    if isempty(goodfits)
        grain{ii}.extended = false;
    else
        grain{ii}.extended = true;
        
        fprintf('Pairs added to grain #%d      :   %s\n', ii,...
                num2str(tot.pairid(tryrefids(goodfits))'))
    end
   
    
    % Include good fits in grain, get updated grain data and excluded 
    % reflection indices (if grain is found invalid, excrefids contains all
    % its refid-s)
    [grain{ii}, excrefids] = gtIndexModifyGrain(grain{ii}, [], ...
                             tryrefids(goodfits), tot, tol_b, cryst);
   
    
    if isempty(grain{ii})
        gtodel = [gtodel ii];
        fprintf('Grain #%d was found invalid and deleted.\n', ii)
    end
    
    % Delete the reflections from list that were added to the grain and add
    % back the ones that were rejected:
    tryrefids(goodfits) = [];
    tryrefids = [tryrefids; excrefids];
    
end

% Remaining reflections
remaining = tryrefids;

% Delete bad grains
grain(gtodel) = [];


disp(' ')
toc

end
																			
																			
