function [bestSolution, bestSquareRes, jj] = gtMathsCGSolveCell(A, b, toll, maxIter, verbose, use_c_functions)
% GTMATHSCGSOLVECELL Conjugate Gradient solver for Cell arrays
%   [z] = gtMathsCGSolveCell(A, b, maxIter[, verbose])

    if (~exist('verbose', 'var'))
        out = GtConditionalOutput(false);
    else
        out = GtConditionalOutput(verbose);
    end
    if (~exist('use_c_functions', 'var'))
        use_c_functions = false;
    end

    if (~iscell(b))
        error('gtMathsCGSolveCell:wrong_argument', ...
              'This function works on Cell arrays');
    end
    if (~isa(A, 'function_handle'))
        if (iscell(A) && (length(A) == length(b)))
%             A = @(x)cellfun(@(M, y){M * y}, A, x);
            A = @(x)multiplyCellMatrix(A, x);
        elseif (isnumeric(A))
            warning('gtMathsCGSolveCell:parameter', ...
                    ['I''m assuming that the input A is a square matrix, that'
                     ' suits for the vectors in  b']);
%             A = @(x)cellfun(@(y){A * y}, x);
            A = @(x)multiplyMatrix(A, x);
        end
    end

    numCells = length(b);

    % To see the trend of residuals
    squareResiduals = zeros(maxIter, 1);

    % Solution to the system
    z = cell(1, numCells);
    for ii = 1:numCells
        z{ii} = zeros(size(b{ii}));
    end
    % Initial residual which is exactly the b vector, since z = 0
    if (use_c_functions)
        r = internal_cell_copy(b);
    else
        r = b;
    end
    % First correction is gradient step
    if (use_c_functions)
        d = internal_cell_prod_scalar_copy(r, -1.0);
    else
        d = cell(1, numCells);
        for ii = 1:numCells
            d{ii} = -r{ii};
        end
    end
    nextSquareResidual = squareNorm(r, use_c_functions);

    % Best solutions in the iterations (just in case we run till the end)
    bestSquareRes = nextSquareResidual;
    bestSolution = d;

    % Tollerance to terminate the algorithm
    toll = (getTollerance(r, nextSquareResidual, use_c_functions) * toll / numCells) ^ 2;
%     toll = (toll / numCells) ^ 2;

    for jj = 1:maxIter
        % Next iteration
        squareResiduals(jj) = nextSquareResidual;

        % The vector multiplied by the matrix
        q = A(d);

        % d' * A * d
        squareAd = dotProduct(d, q, use_c_functions);
        if (squareAd < 0)
            if (jj == 0)
                out.fprintf('\nExit by bad Hessian\n')
                break;
            else
                out.fprintf('\nExit by negativeness\n')
                break;
            end
        end

        % Update the solution
        alpha = squareResiduals(jj) / squareAd;
        if (use_c_functions)
            z = internal_cell_sub_scaled_assign(z, d, alpha);
        else
            for ii = 1:numCells
                z{ii} = z{ii} - alpha * d{ii};
            end
        end

        % Evaluation of the residual
        if (mod(jj+1, 50) == 0)
            % Evaluation of the true residual
            tempAz = A(z);
            if (use_c_functions)
                r = internal_cell_sub_copy(b, tempAz);
            else
                for ii = 1:numCells
                    r{ii} = b{ii} - tempAz{ii};
                end
            end
        else
            % Let's update the residual
            if (use_c_functions)
                r = internal_cell_sum_scaled_assign(r, q, alpha);
            else
                for ii = 1:numCells
                    r{ii} = r{ii} + alpha * q{ii};
                end
            end
        end

        % computing beta and the weights
        nextSquareResidual = squareNorm(r, use_c_functions);
        beta = nextSquareResidual / squareResiduals(jj);

        % Test if it is the best, and save it
        if (nextSquareResidual < bestSquareRes)
            bestSquareRes = nextSquareResidual;
            bestSolution = z;
        end

        out.fprintf('Iteration %03d/%03d: BestResidualSquareNorm %f, ResidualSquareNorm %5.20f\n', ...
                    jj, maxIter, bestSquareRes, nextSquareResidual);

        % Exit condition
        if (nextSquareResidual < toll)
            out.fprintf('\nExit by tollerance\n');
            break;
        end

        % Update the correction to apply at the next iteration
        if (use_c_functions)
            d = internal_cell_prod_scalar_assign(d, beta);
            d = internal_cell_sub_assign(d, r);
        else
            for ii = 1:numCells
                d{ii} = -r{ii} + beta * d{ii};
            end
        end
    end
end

function toll = getTollerance(residual, squareNormGrad, use_c_functions)
    if (~exist('squareNormGrad', 'var'))
        squareNormGrad = squareNorm(residual, use_c_functions);
    end

    if (squareNormGrad < 1)
        normGrad = sqrt(squareNormGrad);
        toll = min(0.5, squareNormGrad) * normGrad;
    else
        normGrad = sqrt(squareNormGrad);
        toll = min(0.5, sqrt(normGrad)) * normGrad;
    end
end

function value = squareNorm(x, use_c_functions)
    if (use_c_functions)
        value = internal_cell_square_norm(x);
%         value = internal_cell_dot_product(x, x);
    else
        value = 0;
        for kk = 1:length(x)
            tempSquare = sum(x{kk} .* x{kk});
            value = value + sum(tempSquare(:));
        end
    end
end

function value = dotProduct(x, y, use_c_functions)
    if (use_c_functions)
        value = internal_cell_dot_product(x, y);
    else
        value = 0;
        for kk = 1:length(x)
            tempSquare = sum(x{kk} .* y{kk});
            value = value + sum(tempSquare(:));
        end
    end
end

function prod = multiplyCellMatrix(M, x)
    numCells = length(M);
    prod = cell(1, numCells);
    for ii = 1:numCells
        prod{ii} = M{ii} * x{ii};
    end
end

function prod = multiplyMatrix(M, x)
    numCells = length(x);
    prod = cell(1, numCells);
    for ii = 1:numCells
        prod{ii} = M * x{ii};
    end
end
