function euler = gtMathsRod2Euler(r)
% GTMATHSROD2EULER  Convert Rodrigues vector r into Euler angles (in degrees).
%                   These Euler angles follows Bunge convention (ZXZ).
%     -------------------------------------------------------------------------
%     [phi1, phi, phi2] = gtMathsRod2Euler(r)
%
%     INPUT:
%       r     = <double>  Rodrigues vector r, 3 components, can be row or 
%                         column vector
%
%     OUTPUT:
%       euler = <double>  row vector containing the 3 Euler angles (Bunge)
%                         Output angles are in degrees and only positive
%
%     Version 001 15-06-2012 by YGuilhem

% Check size of r vector
if and(size(r,1)==3, size(r,2)==1)
    R = r;
elseif and(size(r,1)==1, size(r,2)==3)
    R = r';
else
    Mexc = MException('Rodrigues:wrong_vector_size',['Given vector size is neither 1x3, neither 3x1!']);
    throw(Mexc);
end

% Compute intermediate variables
epsilon = 1.e-9;
if and(abs(R(1))<epsilon, abs(R(2))<epsilon)
    a = pi/4;
else
    a = atan2(R(2), R(1));
end

if abs(R(3))<epsilon
    b = pi/4;
else
    b = atan2(R(3),1);
end

% Compute Euler angles in radians
phi1 = a + b;
phi  = 2*atan2(sqrt(R(1)^2 +  R(2)^2), sqrt(1 + R(3)^2));
phi2 = b - a;

% Switch to degrees
euler = rad2deg([phi1 phi phi2]);

% Shift to positive angles range
euler = euler + 360.0 * (euler<0);

end % end of function
