function [field,liststruct] = get_structure(par,substr,saveflag,namelist,list)
% GET_STRUCTURE  Gets the field list for each sub structure of par
%     [field,liststruct] = get_structure(par,substr,saveflag,namelist,list)
%     ---------------------------------------------------------------------
%     Returns automatically the list cell-structure of 'par' and prints
%     the syntax in a text file to be used for building the parameters
%     afterwards.
%
%     i.e. [list.acq,tmp] = get_structure(parameters.acq,'acq',false);
%          [list.acq,tmp] = get_structure(parameters.acq,'acq',true,'acq',list);
%          filename   = [datasetname '_list.acq.txt'];
%
%     INPUT:
%       par      = structure to be analysed <struct>
%       substr   = 'par' structure name <string>
%                  i.e. par = parameters.acq  --> substr = 'acq'
%       saveflag = flag to save for each 'struct' field of 'par' a file
%                  named [datasetname]_list.[namelist].txt <logical>
%       namelist = name of the current 'par' structure <string>
%       list     = pre-defined list of {'fieldname','description','class'}
%                  for each field <cell Nx3>
%
%     OUTPUT:
%       field      = built list of {'fieldname','description','class','size(dim1)','size(dim2)'}
%                    for each field <cell Mx5>
%                      PS. each substructure name is saved separately. 
%                          i.e. parameters.index.strategy 
%                                     ---> list.index__strategy 
%                               parameters.index.strategy.b
%                                     ---> list.index__strategy__b
%       liststruct = sub-structures info found in 'par' structure <cell>
%                    liststruct{k}.name <string>
%                    liststruct{k}.subnames <cell>
%                    liststruct{k}.nfields <double>
%                    liststruct{k}.substruct <logical>
%

liststruct = [];
field      = [];

cdir=pwd;
ind=strfind(cdir,'/');
dataset=cdir(ind(end)+1:end);

if ~exist('namelist','var') || isempty(namelist)
    namelist=substr;
end
if ~exist('list','var') || isempty(list)
    list=cell(size(par,1),3); 
end
size(fieldnames(par))
size(list)

names = fieldnames(par);

k=0;
for i=1:length(names)
    if strcmpi(names{i},list{i,1})
        field.(substr){i,1} = list{i,1};
        field.(substr){i,2} = list{i,2};
        if ~isempty(list{i,3})
            field.(substr){i,3} = list{i,3};
        else
            field.(substr){i,3} = class(par.(names{i}));  
        end
    else
        field.(substr){i,1} = names{i};
        field.(substr){i,2} = '(?)';
        field.(substr){i,3} = class(par.(names{i}));
    end
    
    field.(substr){i,4} = size(par.(names{i}),1);
    field.(substr){i,5} = size(par.(names{i}),2);
    if isstruct(par.(names{i}))
        disp(['Found structure: ' names{i}])
        name     = names{i};
        subnames = fieldnames(par.(names{i}));
        nof      = length(subnames);
        is_in=false;
        for j=1:length(subnames)
            if isstruct(par.(names{i}).(subnames{j}))
                is_in=true;
            end
        end
        substruct=is_in;
        k=k+1;
        liststruct{k}.name      = name;
        liststruct{k}.subnames  = subnames;
        liststruct{k}.nfields   = nof;
        liststruct{k}.substruct = substruct;
    end
end

t0=[];
for i=1:size(names,1)
    t=length(names{i});
    t0=[t0;t];
end
maxl=max(t0);
clear t0 t

if saveflag
    sublist=['list.' namelist];
    basename=[dataset '_list.'];
    filename=[basename namelist '.txt'];
    
    
    fid=fopen(filename,'w');
    for i=1:length(names)
        isize=length(field.(substr){i,1});
        rsize=maxl-isize;
        if ~isstruct(par.(names{i}))
            if i>=1 && i<=9 && length(names)>=10
                left='%s{%d,%d}  = ''%s'';';
                column1=sprintf(left,sublist,i,1,char(field.(substr){i,1}));
                column2=sprintf(left,sublist,i,2,char(field.(substr){i,2}));
                column3=sprintf(left,sublist,i,3,char(field.(substr){i,3}));
                fprintf(fid,'%s\n%s\n%s\n',column1,column2,column3);
            else
                left='%s{%d,%d} = ''%s'';';
                column1=sprintf(left,sublist,i,1,char(field.(substr){i,1}));
                column2=sprintf(left,sublist,i,2,char(field.(substr){i,2}));
                column3=sprintf(left,sublist,i,3,char(field.(substr){i,3}));
                fprintf(fid,'%s\n%s\n%s\n',column1,column2,column3);
            end
        end
    end
    disp(['Save list_template in ' filename '.']);
    fclose(fid);
end

field=field.(substr);
    

end % end of function

