#!/usr/bin/python

import re
import fnmatch
import os
import sys
import subprocess
import string

from xml_dct_io import DCTConf

class FunctionsBuilder(object):

    submitMethods = [ 'OAR_make',
                    #  'condor_make' # Disabled since we moved to OAR
                    ]

    def __init__(self, indir = os.getcwd(),
                 script_dir = os.path.join(os.getcwd(), 'bin', 'scripts'),
                 bin_dir = os.path.join(os.getcwd(), 'bin', 'compiled', os.getenv('CPU'))):
        self.indir = indir
        self.script_dir = script_dir
        self.bin_dir = bin_dir
        self.comp_funcs = { }
        self.m_files = { }
        self.compile_records = { }

    def _findFuncs(self):
        searchPatterns = [ ]
        for method in FunctionsBuilder.submitMethods:
            regexpString = '[^%]*\s*' + method + "\s*\(\s*'.*'"
            print('Regular expression: "' + regexpString + '"')
            regexp = re.compile(regexpString)
            searchPatterns.append(regexp)

        print('Finding functions that may be compiled, from: ' + self.indir)
        for root, dir_name, file_names in os.walk(self.indir):
            for file_name in file_names:
                if fnmatch.fnmatch(file_name, '*.m'):
                    filepath = os.path.join(root, file_name)
                    self.m_files[file_name[:-2]] = filepath
                    fid = open(filepath, 'r')
                    filecontent = fid.readlines()
                    for reg in searchPatterns:
                        for line in filecontent:
                            matched = reg.search(line)
                            if matched:
                                function_name = matched.group().split("'")[1]
                                if filepath in self.comp_funcs:
                                    self.comp_funcs[filepath] = \
                                            self.comp_funcs[filepath] | set([function_name])
                                else:
                                    self.comp_funcs[filepath] = set([function_name])
                    fid.close()
        print("Done.\n\nThese functions were found:")
        for file_name in self.comp_funcs:
            print('In file: ' + file_name)
            for func in self.comp_funcs[file_name]:
                print('  Function: ' + func)
        print('')

    def _prepareFuncsList(self):
        print("Adding information about functions that may not need to be compiled..")
        for caller in self.comp_funcs:
            self.compile_records[os.path.basename(caller)] = { }
            for function in self.comp_funcs[caller]:
                outpath = os.path.join(self.bin_dir, function)
                try:
                    self.compile_records[os.path.basename(caller)][function] = (outpath, self.m_files[function])
                except KeyError:
                    print('-> WARNING!! function: "' + function + '" is NOT a local function (may be from ID19) <-')
                    self.compile_records[os.path.basename(caller)][function] = (' ',' ')
        print("Done.")

    def _generateMCompileFile(self):
        if not os.path.exists(self.script_dir):
            print('Creating script directory')
            os.makedirs(self.script_dir)
        print("Generating m-file that will compile those Functions..")
        script_content = [ "function compile(varargin)\n",
                           "current_path = pwd;\n",
                           "compile_path = [ getenv('HOME') '/matlabDCT/' ];\n",
                           "cd(compile_path)\n\n",
                           "error_log = java.util.ArrayList();\n\n",
                           "mfiles = java.util.TreeMap();\n" ]
        for file_name in self.compile_records:
            script_content.append("% From file:" + file_name + "\n")
            script_content.append("mfiles.put('" + file_name + "', java.util.TreeMap());\n")
            for func in self.compile_records[file_name]:
                script_content.append("mfiles.get('" + file_name + "').put('" + func + "',java.util.TreeMap());\n")
                script_content.append("mfiles.get('" + file_name + "').get('" + func + "').put('out_file','" + self.compile_records[file_name][func][0] + "');\n")
                script_content.append("mfiles.get('" + file_name + "').get('" + func + "').put('in_mfile','" + self.compile_records[file_name][func][1] + "');\n")
            script_content.append('\n')
        script_content.append(
"""
function compile_function(functions, funcName, force)
    func = functions.get(funcName);
    disp(['Checking function: "' funcName '":'])
    if (force ...
            || ~gtCheckFunctionUpToDate(func.get('in_mfile'), func.get('out_file')))
        disp('Compiling..')
        if gt_mcc(funcName) == false
            error_log.add(funcName);
        end
    else
        disp('* Is up to date.')
    end
end

function compile_funcs_of_mfile(mfile_name, force)
    if mfiles.containsKey(mfile_name)
        functions = mfiles.get(mfile_name);
        func_keys = java.util.ArrayList(functions.keySet());
        for ii2 = 1:func_keys.size()
            funcName = func_keys.get(ii2-1);
            compile_function(functions, funcName, force);
        end
    else
        disp(['Wrong m-file name: ' mfile_name])
    end
end

if size(varargin,2) == 1 || (size(varargin,2) == 2 && strcmp(varargin{2},'all'))
    keys = java.util.ArrayList(mfiles.keySet());
    for ii = 1:keys.size()
        compile_funcs_of_mfile(keys.get(ii-1), strcmp(varargin{1},'force'));
    end
else
    for ii = 2:size(varargin,2)
        compile_funcs_of_mfile(varargin{ii}, strcmp(varargin{1},'force'));
    end
end

if error_log.size() > 0
  for ii = 0:error_log.size()-1
    disp(['Error while compiling: ' error_log.get(ii)])
  end
end

cd(current_path)

end
""")

        print("Done.\nWriting to disk..")
        script_loc = os.path.join(self.script_dir, 'compile.m')
        fid = open(script_loc, "w")
        fid.writelines(script_content)
        fid.close()
        print("Done.")

    def compileFuncs(self, matlab_executable, mfiles, force):
        if not os.path.exists(self.bin_dir):
            print('Creating binaries directory')
            os.makedirs(self.bin_dir)
        print("Calling matlab..")
        cmd = [matlab_executable]
        cmd = cmd + ['-nodesktop', '-nosplash', '-r']
        args = ''
        if force is True:
            args = "'force'"
        else:
            args = "'no-force'"
        if len(mfiles) is not 0:
            args = args + ",'" + string.join(mfiles,"','") + "'"
        cmd.append('cd ' + self.indir + ';initialise_gt;cd ' + self.script_dir
                    + ';compile(' + args + ');quit;')
        print("Command to execute: " + string.join(cmd," "))
        subprocess.call(cmd)
        print("Done.")

    def generateMfile(self):
        self._findFuncs()
        self._prepareFuncsList()
        self._generateMCompileFile()

    def buildMatlabFunctions(self):
        self.generateMfile()
        self.compileFuncs()


if __name__=="__main__":
    skip_next = False
    will_compile = True
    will_generate = True
    force_compile = False
    matlab_executable = DCTConf().getMatlabCommand()
    mfiles_to_consider = [ ]
    for index in range(len(sys.argv)):
        if skip_next is False:
            if sys.argv[index] == "-just-generate":
                will_compile = False
                will_generate = True
            if sys.argv[index] == "-just-compile":
                will_compile = True
                will_generate = False
            if sys.argv[index] == "-force-compile":
                will_compile = True
                force_compile = True
            if sys.argv[index] == "-c":
                skip_next = True
                if len(sys.argv) > (index-1):
                    mfiles_to_consider.append(sys.argv[index+1])
                else:
                    raise ValueError("Not enough arguments for '-c' option")
            if sys.argv[index] == "-matlab":
                skip_next = True
                if len(sys.argv) > (index-1):
                    matlab_executable = sys.argv[index+1]
                else:
                    raise ValueError("Not enough arguments for '-matlab' option")
        else:
            skip_next = False

    builder = FunctionsBuilder()

    if will_generate is True:
        builder.generateMfile()
    if will_compile is True:
        builder.compileFuncs(matlab_executable, mfiles_to_consider, force_compile)


