% function gtDrawGrainCubes(grains,sample,numbered,hlight,noaxis)
%
% Draws a figure representing grain size and orientation by cubes 
% (unit cells in a cubic lattice) based on the Rodrigues vectors in the
% grain variable output from indexing.
%
% USAGE  gtDrawGrainCubes(grains,sample)
%        gtDrawGrainCubes(grains([1:10]),sample,1,[1,3,5],1)  
%
% INPUT  grains:   grain structure from indexing
%        sample:   parameters and definition of the sample reference system  
%                   (sample=gtSampleGeoInSampleSystem)
%        numbered: if true, grain id-s are shown
%        hlight:   ID-s of grains to be highlighted
%        noaxis:   if true, axes and beam direction are not shown
%


function gtDrawGrainCubes(grains,sample,numbered,hlight,noaxis)


if ~exist('hlight','var')
  hlight=[];
end
if ~exist('numbered','var')
  numbered=[];
end
if ~exist('noaxis','var')
  noaxis=false;
end

nof_grains=length(grains);

figure
view(3);

axhor=10*floor((sample.rad+30)/10);
axver=10*floor((max(sample.top,sample.bot)+50)/10);

axis([-axhor axhor -axhor axhor -axver axver])
axis equal;
hold on

if ~noaxis
	xlabel('X')
	ylabel('Y')
	zlabel('Z')
	plot3([-axhor axhor],[0 0],[0 0],'r-','Linewidth',2)
end

if noaxis
	set(gca,'xtick',[],'xcolor','w')
	set(gca,'ytick',[],'ycolor','w')
	set(gca,'ztick',[],'zcolor','w')
end

t=1:360;
circx=cosd(t)*sample.rad;
circy=sind(t)*sample.rad;
circbotz(1:360)=sample.bot;
circtopz(1:360)=sample.top;
plot3(circx,circy,circbotz,'k-','Linewidth',1)
plot3(circx,circy,circtopz,'k-','Linewidth',1)
plot3([0 0],[0 0],[sample.bot sample.top],'k.-.','Linewidth',1)
plot3([-sample.rad -sample.rad],[0 0],[sample.bot sample.top],'k.-','Linewidth',1)
plot3([0 0],[-sample.rad -sample.rad],[sample.bot sample.top],'k.-','Linewidth',1)
plot3([sample.rad sample.rad],[0 0],[sample.bot sample.top],'k.-','Linewidth',1)
plot3([0 0],[sample.rad sample.rad],[sample.bot sample.top],'k.-','Linewidth',1)


cube_vertices=[0 0 0; 1 0 0; 1 1 0; 0 1 0; 0 0 1; 1 0 1; 1 1 1; 0 1 1]-0.5*repmat([1 1 1],8,1);
cube_faces=[2 3 7 6; 4 1 5 8; 1 2 6 5;  3 4 8 7;  5 6 7 8; 1 2 3 4];
% directions: +yz      -yz      -xz       +xz       +xy      -xy       


for i=1:nof_grains
	if ismember(i,hlight)
		hl=1;
	else
		hl=false;
	end
	sfPlotGrainCube(grains{i},cube_vertices,cube_faces,hl)
	if numbered
		text(grains{i}.center(1),-axhor,grains{i}.center(3),num2str(i),'Color','c')
		text(axhor,grains{i}.center(2),grains{i}.center(3),num2str(i),'Color','c')
	end
end

end


    
function sfPlotGrainCube(grain,cube_vertices,cube_faces,hl)

ga=grain.stat.bbysmean/2;
gc=grain.center;

ggrain=Rod2g(grain.R_vector);
grain_vertices=repmat(gc,8,1)+ga*cube_vertices*ggrain';

if hl
	facecolors=cool(8);
	facecolors=facecolors(end-5:end,:);
else
	facecolors=copper(12);
	facecolors=facecolors(end-1-5:end-1,:);
	%facecolors=gray(12);
	%facecolors=facecolors(end-1-5:end-1,:);
end

patch('Vertices',grain_vertices,'Faces',cube_faces,...
	'FaceVertexCData',facecolors,'FaceColor','flat');
      
end


