function [grain, copl, out] = gtStrainGrainsGeneral(grain, cryst, ...
                              energy, Bmat, errfilt, stdlim)
% GTSTRAINGRAINSGENERAL Elastic strain fitting for multiple grains.
% 
% grain = gtStrainGrainsGeneral(grain, cryst, energy, Bmat, errfilt, stdlim)
%
% Runs a strain calculation for all the grains in the cell array 'grain{i}',
% and puts the results in the field 'grain{i}.strain'.
%
% It calls 'gtStrainSingleGrain' to do the strain calculations (see that
% help for more details).
%
% Grains with all coplanar plane normals or with outlier strain values will
% have their strain components set to NaN.
%
% It filters outliers based on their smallest or largest principal strain
% component being away from the median value of all grains more than 
% 'stdlim' times the corresponding standard deviation.
%
% OPTIONAL INPUT
%   grain   - multpile grain data in cell array as output from Indexter
%   cryst   - crystallography data as in 4_grains/phase_##/index.mat file 
%             or parameters file
%   energy  - energy of the X-ray beam in keV
%   Bmat    - matrix to transform HKL indices into Cartesian coordinates
%   errfilt - error filter for pairs (0 < errfilt <= 1); this best fraction
%             of Friedel pairs will be used for error fitting; 
%             e.g. 0.75 meaning the best 75% based on their path distance 
%             from the grain center of mass; (default is 0.75)
%   stdlim  - limits in std of all grains' smallest and largest 
%             principal strain components to reject outlier grains
%             (default is Inf)
%
%   Either 'grain.hklsp' or 'cryst.shklfam' field is required.
%
% OUTPUT
%   grain.strain - new field added containing strain data for each grain
%   copl         - indices of grains having plane normals that are all
%                  close to being coplanar
%   out          - grain indices that have their principal strain components
%                  identified as outliers
%
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic

if ~exist('grain','var') || isempty(grain)
    disp('Loading grain info from file 4_grains/phase_01/index.mat.')
    index = load('4_grains/phase_01/index.mat');
    grain = index.grain;
end

if ~exist('cryst','var') || isempty(cryst)
    disp('Loading crystallographic info from file 4_grains/phase_01/index.mat.')
    index = load('4_grains/phase_01/index.mat');
    cryst = index.cryst;
end

if ~exist('energy','var') || isempty(energy)
    disp('Loading beam energy from file parameters.mat.')
    parameters = load('parameters.mat');
    parameters = parameters.parameters;
    energy = parameters.acq.energy;
end

if ~exist('Bmat','var') || isempty(Bmat)
    Bmat = gtCrystHKL2CartesianMatrix(cryst.latticepar);
end

if ~exist('errfilt','var') || isempty(errfilt)
    errfilt = 0.75;
end

if ~exist('stdlim','var') || isempty(stdlim)
    stdlim = Inf;
end


lambda = gtConvEnergyToWavelength(energy);

fprintf('Using the parameters:\n')
fprintf('  Beam energy (keV):     %g\n', energy)
fprintf('  Beam wavelength (A):   %g\n', lambda)
fprintf('  Error filter:          %g%%\n', errfilt*100)
fprintf('  Strain outliers (std): %g\n', stdlim)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Run strain fitting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Running strain fitting...')

parfor ii = 1:length(grain)
    
    % Strain fitting (returns 'strain' structure)
    grain{ii}.strain = gtStrainSingleGrain(grain{ii}, cryst, lambda, Bmat, errfilt);
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Filter outliers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Grains with non coplanar plane normals
ok = ~gtIndexAllGrainValues(grain,'strain','coplanar',1);

% Statistics on principal strain values
med_psmax = median(gtIndexAllGrainValues(grain(ok),'strain','pstrain',1));
med_psmin = median(gtIndexAllGrainValues(grain(ok),'strain','pstrain',3));

std_psmax = std(gtIndexAllGrainValues(grain(ok),'strain','pstrain',1));
std_psmin = std(gtIndexAllGrainValues(grain(ok),'strain','pstrain',3));

% Grains with strain uut of range
psout = ((gtIndexAllGrainValues(grain,'strain','pstrain',1) > med_psmax + stdlim*std_psmax) | ...
         (gtIndexAllGrainValues(grain,'strain','pstrain',3) < med_psmin - stdlim*std_psmin));


for ii = 1:length(grain)
    
    if (grain{ii}.strain.coplanar || psout(ii))        
        
        grain{ii}.strain.strainT(:)    = NaN;
        grain{ii}.strain.Eps(:)        = NaN;
        grain{ii}.strain.pstrain(:)    = NaN;
        grain{ii}.strain.pstraindir(:) = NaN;
        %grain{ii}.strain.coplanar      = NaN;
        %grain{ii}.strain.plfitted      = NaN;
    end
    
    if grain{ii}.strain.coplanar
        fprintf('Planes are coplanar in grain no.    %d\n',ii)
    end
    
    if psout(ii)
        fprintf('Strain is out of range in grain no. %d\n',ii)
    end
    
end


% Grain indices with coplanar plane normals
copl = find(gtIndexAllGrainValues(grain,'strain','coplanar',1));

% Grain indices with strain out of range
out  = find(psout);


toc


end % of function