function [vol, info] = gtTIFVolReader(filename, varargin)
% GTTIFVOLREADER  Read TIFF volume (stack) and store it into a matrix [X,Y,Z].
%     -------------------------------------------------------------------------
%     [vol, info] = gtTIFVolReader(filename, varargin)
%
%     INPUT:
%       filename   = <string>  Path to TIFF file to write
%                              it may be relative or absolute
%                              if extension is missing, '.tif' is append
%
%     OPTIONAL INPUT (varargin as a list of pairs, see parse_pv_pairs.m):
%       xrange     = <range>   Index range of output in X direction
%       yrange     = <range>               "            Y     "
%       zrange     = <range>               "            Z     "
%
%     OUTPUT:
%       vol        = <3Dimage> Output 3D volume
%       info       = <struct>  Information about volume
% 
%     Version 001 15-02-2012 by YGuilhem, yoann.guilhem@esrf.fr

% Set default parameters and parse optional arguments
params.xrange = 'all';
params.yrange = 'all';
params.zrange = 'all';
params = parse_pv_pairs(params, varargin);

if ~exist('filename','var')
    [tmp1, tmp2] = uigetfile('*','Select a .tif volume file');
    filename = fullfile(tmp2,tmp1);
end

[fpath, fname, fext] = fileparts(filename);

% Checking existence of file extension, if not, add it
if ~length(fext)
    filename = [filename '.tif'];
end

% Check existence of file
if ~exist(filename)
    Mexc = MException('TIFF:wrong_file_name',['File ''' filename ''' does not exist!']);
    throw(Mexc);
end

try
    tifInfo = imfinfo(filename);
catch Mexc
    disp('Unable to read information of selected volume.')
    disp('Please check it exists in the directory.')
    disp(' ')
    Mexc1 = MException('TIFF:wrong_input', 'Unable to read file');
    Mexc1 = addCause(Mexc1, Mexc);
    throw(Mexc1);
end

fullFilename = tifInfo(1).Filename;
[fpath, fname, fext] = fileparts(fullFilename);
tifSizeX = tifInfo(1).Width;
tifSizeY = tifInfo(1).Height;
tifSizeZ = numel(tifInfo);

if strcmp(params.xrange,'all')
    rangeX = 1:tifSizeX;
else
    rangeX = params.xrange;
end
if strcmp(params.yrange,'all')
    rangeY = 1:tifSizeY;
else
    rangeY = params.yrange;
end
if strcmp(params.zrange,'all')
    rangeZ = 1:tifSizeZ;
else
    rangeZ = params.zrange;
end

volSizeX = length(rangeX);
volSizeY = length(rangeY);
volSizeZ = length(rangeZ);

switch tifInfo(1).BitDepth
    case 1
        ftype = 'logical';
    case 8
        ftype = 'uint8';
    case 12
        ftype = 'uint16';
    case 16
        ftype = 'uint16';
    otherwise
        ftype = '';
end

if ~length(ftype)
    vol = zeros(volSizeX,volSizeY,volSizeZ);
elseif strcmp(ftype,'logical')
    vol = false(volSizeX,volSizeY,volSizeZ,'uint8');
else
    vol = zeros(volSizeX,volSizeY,volSizeZ,ftype);
end

% Same info as HST volumes
info.volSizeX = volSizeX;
info.volSizeY = volSizeY;
info.volSizeZ = volSizeZ;
info.bytespervoxel = tifInfo(1).SamplesPerPixel * tifInfo(1).BitsPerSample / 8;
info.fpath = fpath;
info.fname_final = fullFilename;
info.byteorder = tifInfo(1).ByteOrder(1);
info.ftype = ftype;
info.fname_prefix = fname;

if nargin <= 1
    for i=1:volSizeZ
        vol(:,:,i) = imread(filename, rangeZ(i))';
    end
else
    for i=1:volSizeZ
        vol(:,:,i) = imread(filename, rangeZ(i), 'PixelRegion', {[rangeY(1) rangeY(end)], [rangeX(1) rangeX(end)]})';
        % This second method should be faster but in fact...
        %vol(:,:,i) = imread(filename, rangeZ(i), 'Info', info, 'PixelRegion', {[rangeY(1) rangeY(end)], [rangeX(1) rangeX(end)]})';
    end
end

end % end of function
