function gtTIFVolWriter(vol, filename, varargin)
% GTTIFVOLWRITER  Writes volume data to TIFF file without compression.
%                 Either single file of multiple files stack can be produced.
%                 The produced TIFF files contain 8/16 bit unsigned integers.
%     -------------------------------------------------------------------------
%     gtTIFVolWriter(vol, filename, varargin)
% 
%     INPUT:
%       vol        = <3Dimage> Input 3D volume
%       filename   = <string>  Path to TIFF file to write
%                              The path may be relative or absolute.
%                              If extension is missing, '.tif' will be append.
%                              In the case of stack output, this will be the 
%                              stack filename prefix to which the slice number 
%                              and the '.tif' extension will be append.
%
%     OPTIONAL INPUT (varargin as a list of pairs, see parse_pv_pairs.m):
%       xrange     = <range>   Index range of output in X direction
%       yrange     = <range>               "            Y     "
%       zrange     = <range>               "            Z     "
%
%       type       = <string>  Type of data written in the TIFF file
%                              it can be 'uint8' or 'uint16'
%                              default is 'uint8'
%
%       output     = <string>  Output type which is either 'single' or 'stack'
%                              default is a single TIFF file
%       digits     = <int>     Number of digits in stack filenames
%       startindex = <int>     Starting index for stack filenames
%
%     TODO :
%     - RGB data output
%
%     Version 002 19-07-2012 by YGuilhem, yoann.guilhem@esrf.fr
%         Add the options to output the volume as a stack of multiple TIFF
%         files.
%
%     Version 001 15-02-2012 by YGuilhem, yoann.guilhem@esrf.fr

% Set default parameters and parse optional arguments
params.type = 'uint8';
params.xrange = 'all';
params.yrange = 'all';
params.zrange = 'all';
params.output = 'single';
params.digits = 4;
params.startindex = 1;
params = parse_pv_pairs(params, varargin);

% Translating range parameters
if strcmp(params.xrange,'all')
    rangeX = 1:size(vol,1);
else
    rangeX = params.xrange;
end
if strcmp(params.yrange,'all')
    rangeY = 1:size(vol,2);
else
    rangeY = params.yrange;
end
if strcmp(params.zrange,'all')
    rangeZ = 1:size(vol,3);
else
    rangeZ = params.zrange;
end
outVolSizeX = length(rangeX);
outVolSizeY = length(rangeY);
outVolSizeZ = length(rangeZ);

if ~isa(vol,params.type)
    % Creating grayscale integer volume 
    grayVol = mat2gray(vol);
    switch params.type
        case 'uint8' 
            outVol = uint8(grayVol*255);
        case 'uint16'
            outVol = uint16(grayVol*65535);
        otherwise
            Mexc = MException('TIFF:wrong_type', ['Output type ''' type ''' is not supported!']);
            throw(Mexc);
    end
else
    outVol = vol;
end

% Guessing output directory, filename and extension
[fdir, fname, fext] = fileparts(filename);

if strcmpi(params.output, 'single')

    % Setting output filename
    if ~length(fext)
        filename = fullfile(fdir, [fname '.tif']);
    end

    % Check existence of output file to delete it before writing
    if exist(filename, 'file')
        delete(filename);
    end

    % Writing volume in TIFF file
    for i=1:length(rangeZ)
        imwrite(outVol(rangeX(1):rangeX(end), rangeY(1):rangeY(end),rangeZ(i))', filename, 'tif', 'Compression', 'none', 'writemode', 'append');
    end

elseif strcmpi(params.output, 'stack')

    % Check stack starting index
    if isnumeric(params.startindex)
        if mod(params.startindex,1) ~= 0
            params.startindex = round(params.startindex);
            disp(['Argument ''startindex'' convert to ' num2str(params.startindex)]);
        end
    else
        Mexc = MException('INPUT:wrong_type', ['The argument ''startindex'' should be an integer!']);
        throw(Mexc);
    end

    % Check stack number of digits
    if isnumeric(params.digits)
        if mod(params.digits,1) ~= 0
            params.digits = round(params.digits);
            disp(['Argument ''digits'' convert to ' num2str(params.digits)]);
        end
    else
        Mexc = MException('INPUT:wrong_type', ['The argument ''digits'' should be an integer!']);
        throw(Mexc);
    end

    % Create directory
    if and(length(fdir), ~isdir(fdir))
        mkdir(fdir);
    end

    % Output file output format
    fprefix = fullfile(fdir, fname);
    filenameFormat = sprintf('%s%%0%dd.tif', fprefix, params.digits);

    % Set output index offset
    offset = params.startindex - 1;

    % Writing volume in TIFF file
    for i=1:length(rangeZ)
        outFile = sprintf(filenameFormat, i+offset);
        imwrite(outVol(rangeX(1):rangeX(end), rangeY(1):rangeY(end),rangeZ(i))', outFile, 'tif', 'Compression', 'none');
    end

end

end % end of function
