function gtVolView(vol, varargin)
% GTVOLVIEW  A simple viewer of 3D volumes. Not stable, use GtVolView instead.
%            Unless you want to use the linkname functionality. 
%     -------------------------------------------------------------------------
%     gtVolView(vol, varargin)
%
%     INPUT:
%       vol          = <3Dimage>       Input image to view
%
%     OPTIONAL INPUT (varargin as a list of pairs, see parse_pv_pairs.m):
%       'plane'      = 'xy','yz','zx'  Set the viewing plane
%       'slicendx'   = <int>           Start viewing at slice index supplied
%       'cmap'       = <string>        Set the colour map ('gray','jet', etc).
%                                      See 'help graph3d'
%                                      The cmap_saturation colormap is quite
%                                      useful for determining which pixels
%                                      would be saturated for certain min/max
%                                      default is 'gray'
%       'cbar'       = <bool>          Show colourbar
%       'clims'      = perc,[min max]  Set the colour limits by giving either 
%                                      a single value or two values
%                                      see 'help gtSetColourLimits'
%       'hordisplay' = [min max]       Set limits for windows horizontal axis
%       'verdisplay' = [min max]                 "             vertical   "
%       'parent'     = <ui_handle>     You can supply a graphic object where 
%                                      the volume viewer will install into
%       'overlay'    = <vol>           If you supply a single image with the
%                                      dimensions as the slice you are looking
%                                      at, it is overlaid on top.  Watch your
%                                      colormap!
%       'linkname'   = something       If you run gtVolView multiple times,
%                                      using the same linkname, the sliders
%                                      are linked together in the different
%                                      viewers.  What fun!
%
%     OUTPUT:
%       h           = <ui_handle>      Graphical user interface handle   
%
%     EXAMPLES:
%       gtVolView(vol,'cbar',true,'plane','yz','slicendx',43,'cmap',jet,'clims',3)
%
%       You can use vol = gtHSTVolReader to load your ESRF HST volumes.
%
%     SUB-FUNCTIONS:
%[sub]- sfInitialise
% 
%     Version 003 10-02-2012 by NVigano, nicola.vigano@esrf.fr
%       Imported and modified from vol_view.m (id19)
%
%     Version 002 06-06-2008 by AKing,   andrew.king@esrf.fr
%       Changed how 'yz' and 'zx' viewing planes are defined to be
%       more applicable to tomography data: keep z axis vertical in these views.
%       Note I have *NOT* implemented this for the inactive memory mapping
%       options.
%       Changed default 'clims' to 0: quicker and useful for certain colourmaps.
%
%     Version 001 XX-XX-XXXX by AKing,   andrew.king@esrf.fr
%       vol_view.m

warning('This function has stability problems. Use GtVolView instead, unless you need the linkname funcionality.')

    app = sfInitialise(vol, [], varargin);
    app = initFigure(app);
    updateDisplay(0, 0, app.h_figure);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function conf = sfInitialise(vol, prevconf, arguments)
        if isa(vol, 'char') % user has specified a filename
            vol = gtHSTVolReader(vol);
        else
            if islogical(vol)
                disp('Converting logical volume to uint8 for display')
                vol = uint8(vol);
            end
        end

        if isempty(prevconf)
            appdefaults.slicendx = 1;
            appdefaults.clims = 0;
            appdefaults.cbar = false;
            appdefaults.cmap = 'gray';
            appdefaults.plane = 'xy';

            appdefaults.linkname = [];
            appdefaults.overlay = [];
            appdefaults.parent = [];
        else
            appdefaults = prevconf;
        end

        appdefaults.dim = size(vol);
        appdefaults.hordisplay = [1 appdefaults.dim(2)];
        appdefaults.verdisplay = [1 appdefaults.dim(1)];

        conf = parse_pv_pairs(appdefaults, arguments);

        % Internal parameters (not exposed to the user)
        conf.firsttime = true;
        conf.vol = vol;

        % modify the colour limits if a percentile is requested
        conf.clims = gtSetColourLimits(conf.vol, conf.clims, true);

        % Process the color map
        conf.cmap = gtSetColourMap(conf.vol, conf.cmap, true);

        % now validate the arguments
        % ensure slice is possible for a given orientation
        switch conf.plane
            case 'xy'
            case 'zx' % was 'yz' for tmpdim == 1 (y) should be zx
                % to put dim 3 vertical, need to change these around
                if (conf.hordisplay == appdefaults.hordisplay)
                    % user did not change the display scale
                    conf.hordisplay = [1 conf.dim(2)];
                end
                if (conf.verdisplay == appdefaults.verdisplay)
                    % user did not change the display scale
                    conf.verdisplay = [1 conf.dim(3)];
                end
            case 'yz' % was 'zx' for tmpdim == 2 (x) should be yz
                % to put dim 3 vertical, need to change these around
                if (conf.hordisplay == appdefaults.hordisplay)
                    % user did not change the display scale
                    conf.hordisplay = [1 conf.dim(1)];
                end
                if (conf.verdisplay == appdefaults.verdisplay)
                    % user did not change the display scale
                    conf.verdisplay = [1 conf.dim(3)];
                end
            otherwise
                help(mfilename)
                Mexc = MException('GUI:wrong_argument', ...
                                  'Plane must be one of ''xy'',''yz'',''zx''');
                throw(Mexc);
        end
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function conf = initFigure(conf)
        if (isempty(conf.parent))
            conf.h_figure = figure( 'Name', 'Volume Viewer', ...
                                    'NumberTitle', 'off', 'MenuBar', 'none', ...
                                    'Toolbar', 'none');
            set(conf.h_figure, 'CloseRequestFcn', {@guiQuit, conf.h_figure});
            currentParent = conf.h_figure; % Where are incapsulating in
        else
            conf.close_callback = @guiQuit;
            conf.h_figure = gtGetParentFigure(conf.parent);
            currentParent = conf.parent;
        end
        %set(gcf,'handlevisibility','callback')  % protect the window from being accidentall reused
        zoomOp = zoom(conf.h_figure);
        zoomState = get(zoomOp, 'Enable');
        zoom(conf.h_figure, 'off');

        conf.main_boxes = uiextras.VBox('Parent', currentParent);
        conf.upper_boxes = uiextras.HBox('Parent', conf.main_boxes);
        conf.center_boxes = uiextras.HBox('Parent', conf.main_boxes);

        conf.h_sliceslabel = uicontrol('Style', 'text', ...
                                       'Parent', conf.main_boxes, ...
                                       'String', 'Slide through the slices');
        set(conf.h_sliceslabel, 'units', 'normalized');

        conf.h_slider = uicontrol('Style', 'slider', 'Parent', conf.main_boxes);
        set(conf.h_slider, 'units', 'normalized');
        set(conf.h_slider, 'Tag', 'slider');

        set(conf.main_boxes, 'Sizes', [20 -1 15 20]);

        if (conf.dim(3) == 1)
            set(conf.h_slider, 'Visible', 'off');
        end

        %%% Brightness / Contrast
        conf.h_min = uicontrol('Style', 'slider', 'Parent', conf.center_boxes);
        set(conf.h_min, 'Units', 'normalized');

        conf.h_max = uicontrol('Style', 'slider', 'Parent', conf.center_boxes);
        set(conf.h_max, 'Units', 'normalized');

        conf.h_minmaxlabel = uicontrol('Style', 'text', ...
                                       'Parent', conf.upper_boxes, ...
                                       'HorizontalAlignment', 'left');
        set(conf.h_minmaxlabel, 'units', 'normalized');
        %%% Done Brightness - Contrast

        conf.display_boxes = uiextras.VBox('Parent', conf.center_boxes);
        set(conf.center_boxes, 'Sizes', [20 20 -1]);

        conf.h_ax = axes('Parent', conf.display_boxes, ...
                         'ActivePositionProperty', 'OuterPosition');
        conf.h_im = imagesc('Parent', conf.h_ax);

        % Set up the overlay
        if ~isempty(conf.overlay)
            hold(conf.h_ax, 'on');
            conf.h_overlay = imagesc(conf.overlay, 'Parent', conf.h_ax);
            hold(conf.h_ax, 'off');

            conf.h_translabel = uicontrol('Style', 'text', ...
                                          'Parent', conf.display_boxes, ...
                                          'String', 'Overlay transparency');
            % adds brightness and contrast sliders to the figure containing the
            % specified axis
            conf.h_trans = uicontrol('Style', 'slider', ...
                                     'Parent', conf.display_boxes, ...
                                     'Min', 0, 'Max', 1, 'Value', 0.5);
            set(conf.h_trans, 'Units', 'normalized');

            set(conf.display_boxes, 'Sizes', [-1 15 20]);

            set(conf.h_figure, 'Renderer', 'OpenGL');
%             set(conf.h_figure, 'Renderer', 'zbuffer');
        end

        conf.pixelinfo_panel = uipanel('Parent', conf.upper_boxes, ...
                                       'BorderType', 'none');
        % must disable zoom before setting impixelinfoval (R2008A)
        conf.h_pixelinfo = impixelinfoval(conf.pixelinfo_panel, conf.h_im);
        set(conf.h_pixelinfo, 'Unit', 'normalized');
        set(conf.h_pixelinfo, 'Position', [0 0 1 1]);

        conf.h_zoom_button = uicontrol('Parent', conf.upper_boxes, ...
                                       'Style', 'togglebutton', ...
                                       'String', 'Zoom');
        if strcmp(zoomState, 'on')
            set(conf.h_zoom_button, 'Value', get(conf.h_zoom_button, 'Max'));
        else
            set(conf.h_zoom_button, 'Value', get(conf.h_zoom_button, 'Min'));
        end
        set(conf.h_zoom_button, 'Callback', {@updateZoom, conf.h_figure});

        set(conf.upper_boxes, 'Sizes', [-1 -1 50]);

        % Reset back the zoom to the previous state (before the inclusion)
        set(zoomOp, 'Enable', zoomState);

        % write the app variable to a shared location (appending if necessary)
        if ~isempty(conf.linkname)
            if (isappdata(0, conf.linkname))
                appdata = getappdata(0, conf.linkname);
                % check that sliders have same maximum values
                if get(appdata(end).app.h_slider, 'max') == get(conf.h_slider, 'max')
                    disp(['Adding instance ' num2str(length(appdata)+1)])
                    appdata(end+1).app = conf;
                    linkaxes([conf.h_ax, appdata(1).app.h_ax], 'xy')
                else
                    disp('Cannot add using this linkname - the volumes do not have the same')
                    disp('number of slices')
                    conf.linkname = [];
                end
            else
                disp('First instance')
                appdata(1).app = conf;
            end
            setappdata(0, conf.linkname, appdata);
        end

        % Finally add the Callback to update the volume (not a GUI Callback)
        conf.changeVolume_Callback = @changeVol;

        % Stores the datastructure to the figure
        setappdata(conf.h_figure, 'gtVolViewConf', conf);

        % Once we have the structures registered into the guidata, we can set
        % the parameters corresponding to the volume
        resetUiComponents(conf.h_figure);

        %%% GUI Callbacks need to be added later on because the addition of them
        %%% triggers their call, and if the application data is not saved yet,
        %%% matlab is going to throw errors in the callbacks.
        %%% Thankfully these callbacks just need the objects but not the other
        %%% callbacks
        slider_cb = findjobj(conf.h_slider, 'Persist');
        set(slider_cb, 'AdjustmentValueChangedCallback', {@updateDisplay, conf.h_figure});

        slider_cb = findjobj(conf.h_min, 'Persist');
        set(slider_cb, 'AdjustmentValueChangedCallback', {@updateBC, conf.h_figure});

        slider_cb = findjobj(conf.h_max, 'Persist');
        set(slider_cb, 'AdjustmentValueChangedCallback', {@updateBC, conf.h_figure});

        if ~isempty(conf.overlay)
            slider_cb = findjobj(conf.h_trans, 'Persist');
            set(slider_cb, 'AdjustmentValueChangedCallback', {@updateOverlay, conf.h_figure});
        end
        %%% End callbacks

        setappdata(conf.h_figure, 'gtVolViewConf', conf);
    end

    function resetUiComponents(gui_ref)
        conf = getappdata(gui_ref, 'gtVolViewConf');

        switch (conf.plane)
            case 'xy'
                set(conf.h_slider, 'min', 1, 'max', conf.dim(3), ...
                    'sliderstep', [1/conf.dim(3) 1/conf.dim(3)]);
            case 'yz' %slider dimension should be x, hence dim 2
                set(conf.h_slider, 'min', 1, 'max', conf.dim(2), ...
                    'sliderstep', [1/conf.dim(2) 1/conf.dim(2)]);
            case 'zx' %slider dimension should be y, hence dim 1
                set(conf.h_slider, 'min', 1, 'max', conf.dim(1), ...
                    'sliderstep', [1/conf.dim(1) 1/conf.dim(1)]);
        end

        set(conf.h_slider, 'Value', conf.slicendx);

        set(conf.h_min, 'min', conf.clims(1), 'max', conf.clims(2), ...
            'Value', conf.clims(1));
        set(conf.h_max, 'min', conf.clims(1), 'max', conf.clims(2), ...
            'Value', conf.clims(2));

        setappdata(conf.h_figure, 'gtVolViewConf', conf);
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function updateDisplay(~, ~, gui_ref)
        conf = getappdata(gui_ref, 'gtVolViewConf');

        if (isequal(gco, conf.h_slider) || conf.firsttime)
            sliderVal = round(get(conf.h_slider, 'Value'));
            set(conf.h_slider, 'Value', sliderVal);
            if (sliderVal ~= conf.slicendx || conf.firsttime)  % slice change
                conf.slicendx = sliderVal;
                set(conf.h_slider, 'Value', conf.slicendx);
                if (conf.firsttime)
                    switch (conf.plane)
                        case 'xy'
                            slice = conf.vol(:,:,1);
                        case 'yz' %changed these around and added transpose
                            slice = squeeze(conf.vol(:,1,:))';
                        case 'zx' %changed these around and added transpose
                            slice = squeeze(conf.vol(1,:,:))';
                    end
                    xlim = [0.5 size(slice,2)+0.5];
                    ylim = [0.5 size(slice,1)+0.5];
                else
                    xlim = get(conf.h_ax, 'xlim');
                    ylim = get(conf.h_ax, 'ylim');
                end

                switch (conf.plane)
                    case 'xy'
                        set(conf.h_im, 'cdata', conf.vol(:,:,conf.slicendx));
                    case 'yz'  %yz slice is vol(:,i,:) - ie slice is in x dim 2
                        slice_yz = squeeze(conf.vol(:, conf.slicendx,:));
                        % I want dim 3, z, to be vertical
                        slice_yz = slice_yz';
                        set(conf.h_im, 'cdata', slice_yz);
                    case 'zx'
                        slice_zx = squeeze(conf.vol(conf.slicendx,:,:));
                        % I want dim 3, z, to be vertical - this also needs a flip
                        slice_zx = fliplr(slice_zx');
                        set(conf.h_im, 'cdata', slice_zx);
                end

                axis(conf.h_ax, 'on');
                axis(conf.h_ax, 'equal');
                axis(conf.h_ax, 'ij');

                switch (conf.plane)
                    case 'xy'
                        xlabel('X');
                        ylabel('Y');
                        set(conf.h_im, 'xdata', linspace(conf.hordisplay(1), conf.hordisplay(2), conf.dim(2)))
                        set(conf.h_im, 'ydata', linspace(conf.verdisplay(1), conf.verdisplay(2), conf.dim(1)))
                    case 'yz'
                        xlabel('Y');
                        ylabel('Z');
                        set(conf.h_im, 'xdata', linspace(conf.hordisplay(1), conf.hordisplay(2), conf.dim(1)))
                        set(conf.h_im, 'ydata', linspace(conf.verdisplay(1), conf.verdisplay(2), conf.dim(3)))
                    case 'zx'
                        xlabel('X');
                        ylabel('Z');
                        set(conf.h_im, 'xdata', linspace(conf.hordisplay(1), conf.hordisplay(2), conf.dim(2)))
                        set(conf.h_im, 'ydata', linspace(conf.verdisplay(1), conf.verdisplay(2), conf.dim(3)))
                end

                set(conf.h_ax, 'xlim', xlim); % reset the axis limits to those of the previous image (possibly zoomed in)
                set(conf.h_ax, 'ylim', ylim);

                colormap(conf.h_ax, conf.cmap)
                if (conf.cbar), colorbar('peer', conf.h_ax); end
                title(conf.h_ax, sprintf('Slice: %3d', conf.slicendx));

                conf.firsttime = false;

                if ~isempty(conf.linkname)
                    % Get global apps data
                    apps = getappdata(0, conf.linkname);

                    for n = 1:length(apps)
                        tmp_slider = apps(n).app.h_slider;
                        if tmp_slider ~= conf.h_slider
                            currentobject = gco;
                            set(tmp_slider, 'Value', conf.slicendx);
                            tmp_cb = get(tmp_slider, 'Callback');
                            uicontrol(tmp_slider) % focus on the other slider
                            tmp_cb('')  % fire the callback to update the other display
                            uicontrol(currentobject) % return the focus to the original object
                        end
                    end
                    drawnow
                end
            end
        end

        % Needed in order to 'remember' that the flag 'firsttime' is now false
        setappdata(gui_ref, 'gtVolViewConf', conf);

        % Let's trigger also the others
        updateBC(0, 0, gui_ref);
        if (~isempty(conf.overlay)), updateOverlay(0, 0, gui_ref); end
    end

    function guiQuit(~, ~, gui_ref)
        conf = getappdata(gui_ref, 'gtVolViewConf');

        if ~isempty(conf.linkname)
            apps = getappdata(0,conf.linkname);
            for n = 1:length(apps)
                if apps(n).app.h_figure == conf.h_figure
                    apps(n) = [];
                    % disp('Removing this instance from links')
                    break
                end
            end
            if isempty(apps)
                rmappdata(0, conf.linkname)
            else
                setappdata(0, conf.linkname, apps)
            end
        end

        rmappdata(gui_ref, 'gtVolViewConf');

        if ~isempty(conf.parent)
            try
                delete(conf.h_figure)
            catch Mexc
                gtPrintException(Mexc, 'Problem in closing figure');
            end
        end
    end

    function updateBC(~, ~, gui_ref)
    % GTVOLVIEW/UPDATEBC Function to update Brightness and Contrast
    %
    % TODO should have a guard here to prevent it running for all mouse
    % movements
        conf = getappdata(gui_ref, 'gtVolViewConf');

        clims = [get(conf.h_min, 'Value') get(conf.h_max, 'Value')];

        if clims(1) >= clims(2)
            if gco == conf.h_max
                set(conf.h_max, 'Value', get(conf.h_min, 'Value'));
            else
                set(conf.h_min, 'Value', get(conf.h_max, 'Value'));
            end
            clims(2) = clims(1)+1;
        end

        set(conf.h_ax, 'clim', clims)
        set(conf.h_minmaxlabel, 'string', sprintf('Limits: [%.6g, %.6g]', clims));
        drawnow;
    end

    function updateOverlay(~, ~, gui_ref)
    % GTVOLVIEW/UPDATEOVERLAY Function to update the Overlay Transparency
    %
    % TODO should have a guard here to prevent it running for all mouse
    % movements
        conf = getappdata(gui_ref, 'gtVolViewConf');

        alphaval = get(conf.h_trans, 'Value');
        set(conf.h_overlay, 'alphadata', alphaval);

        drawnow;
    end

    function updateZoom(~, ~, gui_ref)
    % GTVOLVIEW/UPDATEOVERLAY Function to update the Overlay Transparency
    %
    % TODO should have a guard here to prevent it running for all mouse
    % movements
        conf = getappdata(gui_ref, 'gtVolViewConf');

        zoomOp = zoom(conf.h_figure);
        value = get(conf.h_zoom_button, 'Value');

        if (value == get(conf.h_zoom_button, 'Min'))
            set(zoomOp, 'Enable', 'off');
        else
            set(zoomOp, 'Enable', 'on');
        end
        drawnow;
    end

    function changeVol(newvol, gui_ref, varargin)
        conf = getappdata(gui_ref, 'gtVolViewConf');

        conf = sfInitialise(newvol, conf, varargin);
        setappdata(conf.h_figure, 'gtVolViewConf', conf);

        resetUiComponents(gui_ref)
        updateDisplay(0, 0, gui_ref);
    end
end
